﻿import mx.utils.Delegate;
/**
 * @author lex@silex-ria.org
 */
class org.lexayo.TransparentWModePatch
{
	/**
	 * true if the browser needs the patch to work properly
	 * mozilla, internet explorer
	 */
	var isPatchNeeded:Boolean = false;
	/**
	 * debug variable
	 */
	static var traceIdx:Number=0;
	/**
	 * store the current TextField
	 */
	var currentTextField_txt;
	/**
	 * variables used to store carret position and selection range in the current TextField
	 */
	var caretIndexStart:Number = -1;
	var caretIndexEnd:Number = -1;
	
	/**
	 * implementation of the singleton pattern
	 * store the singleton instance
	 */
	static var _instance:TransparentWModePatch;
	/**
	 * implementation of the singleton pattern
	 */
	static function getInstance():TransparentWModePatch
	{
		// customTrace("getInstance");
		if (_instance) return _instance;
		return new TransparentWModePatch(true);
	}

	/**
	 * constructor
	 */
	function TransparentWModePatch(isCalledByGetInstance:Boolean)
	{
		// check if getInstance was called (implementation of the singleton pattern)
		if (!isCalledByGetInstance || _instance)
		{
			customTrace("TransparentWModePatch ERROR: Use getInstance to create a new class (implementation of the singleton pattern)");
			throw(new Error("Use getInstance to create a new class (implementation of the singleton pattern)"));
			return;
		}
		// customTrace("TransparentWModePatch ok");
		
		// store the singleton instance
		_instance = this;
		
		// start the process
		// or wait for javascript to set the type of browser
		if (_root.browserTypeTransparentWModePatch)
			turnOnPatch("browserTypeTransparentWModePatch","",_root.browserTypeTransparentWModePatch);
		else
			// wait for javascript to set the type of browser
			_root.watch("browserTypeTransparentWModePatch", Delegate.create(this, turnOnPatch));
	}
	/**
	 * start the patch
	 * called by the system if there is a change in _root.browserTypeTransparentWModePatch (watch)
	 * or directly by the constructor
	 */
	function turnOnPatch(prop, oldVal, newVal)
	{
		if (newVal.indexOf("gecko")>-1 || newVal.indexOf("chrome")>-1 /* || newVal.indexOf("msie")>-1 */)
		{
			isPatchNeeded = true;
		}
		else
			return;
			
		// customTrace("turnOnPatch "+newVal);
		
		// initialize listener for focus changes
		Selection.addListener(this);

		// initialise js communication
		_root.keyboardInputFromJS = "";
		_root.watch("keyboardInputFromJS", Delegate.create(this, keyboardInputChanged));
		
		// initialize the main loop
		/*
		var loopTransparentWModePatch_mc:MovieClip;
		loopTransparentWModePatch_mc = _root.createEmptyMovieClip("loopTransparentWModePatch_mc", _root.getNextHighestDepth());
		loopTransparentWModePatch_mc.onEnterFrame = Delegate.create(this, this.onEnterFrame);
		*/
		_global.getSilex().sequencer.addEventListener("onEnterFrame",Delegate.create(this, this.onEnterFrame));
	}
	/**
	 * handle focus changes
	 */
	function onSetFocus(oldfocus,newfocus) 
	{
		// store new TextField instance
		currentTextField_txt = newfocus;
		// store the new current value
		currentTextField_txt.wmodePatchText = currentTextField_txt.htmlText;
	}
	/**
	 * handle text changes
	 */
	function keyboardInputChanged(prop:String, oldVal:String, newVal:String)
	{
		// do nothing if we are not on an input text field
		if (!currentTextField_txt || (currentTextField_txt && currentTextField_txt.type != "input"))
			return "";

		// customTrace("keyboardInputChanged " + newVal+" - "+String.fromCharCode(Number(newVal))+" - "+(String.fromCharCode(Number(newVal))=="\r"));
		
		// check for double byte in IE
		if (System.capabilities.playerType=="ActiveX" && newVal.length > 1)
		{
			//newVal = newVal.charAt(1);
			// customTrace("keyboardInputChanged double byte in IE");
		}
		
		// if there is a significant change
		if (currentTextField_txt.htmlText!=currentTextField_txt.wmodePatchText && !(Key.isDown(Key.CONTROL) && (String.fromCharCode(Number(newVal))=="c" || String.fromCharCode(Number(newVal))=="v" || String.fromCharCode(Number(newVal))=="x" || String.fromCharCode(Number(newVal))=="z")))
		{
			var initial_str:String = currentTextField_txt.htmlText;
			currentTextField_txt.htmlText = currentTextField_txt.wmodePatchText;
			switch(newVal) 
			{
			case "8":
				// backspace
				deleteBefore();
				break;
			case "12":
				// some artifact that happens when keyboard layouts are being switched
				// do nothing
			case "13":
				// enter
				Selection.setSelection(caretIndexStart,caretIndexEnd);
				currentTextField_txt.replaceText(caretIndexStart,caretIndexEnd,"\n");
				Selection.setFocus(currentTextField_txt);
				Selection.setSelection(caretIndexStart+1,caretIndexStart+1);
				break;
			case "46":
				// suppr
				// delete comes in as 46 but so does "."
				if (initial_str.length < currentTextField_txt.wmodePatchText.length)
				{
					// delete case
					deleteAfter();
					break;
				}
				// dot case
				// => continue to default
			default:
				Selection.setSelection(caretIndexStart,caretIndexEnd);
				currentTextField_txt.replaceText(caretIndexStart,caretIndexEnd,String.fromCharCode(Number(newVal)));
	//			currentTextField_txt.replaceSel(String.fromCharCode(Number(newVal)));
				Selection.setFocus(currentTextField_txt);
				Selection.setSelection(caretIndexStart+1,caretIndexStart+1);
			}
		}
		else
		{
			// copy, paste or anything which doesn't change the text
		}
		// store the new value
		currentTextField_txt.wmodePatchText = currentTextField_txt.htmlText;
		
		// return "" so that keyboardInputFromJS allways equals ""
		return "";
	}
	/**
	 * delete a character after the caret
	 */
	function deleteAfter ()
	{
		if (caretIndexStart == caretIndexEnd)
		{
			caretIndexStart++;
			caretIndexEnd++;
		}
			
		deleteBefore();
	}
	/**
	 * delete a character before the caret
	 */
	function deleteBefore()
	{
		// customTrace("deleteBefore "+caretIndexStart+" -> "+caretIndexEnd);
		if (caretIndexStart == caretIndexEnd && caretIndexStart>0) caretIndexStart--;
		// customTrace("deleteBefore "+caretIndexStart+" -> "+caretIndexEnd+" - "+currentTextField_txt.text.charAt(caretIndexEnd));
		
		if (currentTextField_txt.text.charAt(caretIndexEnd) == "")
		{
			if (currentTextField_txt.text.charAt(caretIndexStart-1) == "")
			{
				// erase all case (and of line and beginning of line)
				currentTextField_txt.text = "";
			}
			else
			{
				// end of line case
				Selection.setSelection(caretIndexStart,caretIndexEnd);
				currentTextField_txt.replaceText(caretIndexStart-1,caretIndexEnd,currentTextField_txt.text.charAt(caretIndexStart-1));
				Selection.setSelection(caretIndexStart,caretIndexStart);
			}
		}
		else
		{
			// not the end of the line case
			Selection.setSelection(caretIndexStart,caretIndexEnd);
			currentTextField_txt.replaceText(caretIndexStart,caretIndexEnd+1,currentTextField_txt.text.charAt(caretIndexEnd));
			Selection.setSelection(caretIndexStart,caretIndexStart);
		}
	}

	/**
	 * store the carret position and selection range in the current TextField
	 */
	function onEnterFrame () {
		
		//customTrace(caretIndexStart+","+caretIndexEnd+","+currentTextField_txt+" - ");

		// do nothing if we are not on an input text field
		if (!currentTextField_txt || (currentTextField_txt && currentTextField_txt.type != "input"))
			return;

		// init currentTextField_txt.wmodePatchText
		if (!currentTextField_txt.wmodePatchText)
			currentTextField_txt.wmodePatchText = currentTextField_txt.htmlText;
		
		// do nothing if we are waiting for js to correct the input which was just made
		if (currentTextField_txt.htmlText == currentTextField_txt.wmodePatchText)
		{
			//customTrace(caretIndexStart+" -> "+caretIndexEnd);
			//store the carret position and selection range in the current TextField
			caretIndexStart = Selection.getBeginIndex();
			caretIndexEnd = Selection.getEndIndex();
			//customTrace(caretIndexStart+","+caretIndexEnd);
		}
	}
	static function customTrace (_str) 
	{
		// silex trace
		_global.getSilex().utils.trace("org.lexayo.TransparentWModePatch "+_str);
	}
}