﻿/*
	this file is part of OOF
	OOF : Open Source Open Minded Flash Components

	OOF is (c) 2008 Alexandre Hoyau and Ariel Sommeria-Klein. It is released under the GPL License:

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License (GPL)
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/

import mx.utils.Delegate;
import org.oof.OofBase; 
import mx.events.EventDispatcher;

/**  base class for all oof cell renderers. This is based on the CellRenderer API defined
 * in the mx documentation.
 * 
 * when creating it, be sure to pass the different init elements as an init object as parameter,
 * so that they can be used in the constructor.
 * example :
 * 	var initObj = new Object();
 * 	initObj.listOwner = this;
 * 	initObj.owner = row;
 * 	initObj.getCellIndex = getCellIndex;
 * 	newCell = row.attachMovie(cellRenderer, getNextHighestDepth(), initObj);
 * 
* @author Ariel Sommeria-klein
*/

class org.oof.lists.cellRenderers.CellRendererBase extends mx.core.UIComponent{
	
	var listOwner : MovieClip; // the reference we receive to the list
	var owner:MovieClip; // reference to the row movieclip
	var getCellIndex : Function; // the function we receive from the list
	var	getDataLabel : Function; // the function we receive from the list

	//EventDispatcher
	var addEventListener:Function;
 	var removeEventListener:Function;
    var dispatchEvent:Function;
	
	// UI
	var bg_btn:Button;
	var selected_mc:MovieClip;
	
	//
	var label:String="";
	var data:Object = null;
	var icon:String;

	// constants, override in child class if needed
	var _preferredHeight:Number = 44;
	var _preferredWidth:Number = 44;
	
	//defaults. if listOwner is org.oof.lists.CustomList, they are overridden by values set in the list params.
	var _cellMarginX:Number = 10;
	var _cellMarginY:Number = 10;
	
	private var _firstSetValueDone:Boolean = false;
	
	private var _isSelected:Boolean = false;
	
	/**
	 * constructor
	 * init the background clips
	 */
	function CellRendererBase(Void){
		super();
		EventDispatcher.initialize(this);
		//trace("CellRendererBase createChildren " + this);

		// retrieve click zone
		var click_obj:Object;
		if (bg_btn)
			click_obj = bg_btn;
		else
			click_obj = this;
		
		// attach events
		click_obj.onPress = Delegate.create(this,onPressCallback);
		click_obj.onRelease = Delegate.create(this,onReleaseCallback);
		click_obj.onRollOver = Delegate.create(this,onRollOverCallback);
		click_obj.onRollOut = Delegate.create(this,onRollOutCallback);
		click_obj.onReleaseOutside = Delegate.create(this,onReleaseOutsideCallback);
		click_obj.useHandCursor = listOwner._useHandCursor;
		
		onMouseMove = Delegate.create(this,onMouseMoveCallback);

		if(listOwner.cellMarginX != undefined){
			_cellMarginX = listOwner.cellMarginX;
		}
		if(listOwner.cellMarginY != undefined){
			_cellMarginY = listOwner.cellMarginY;
		}
	}
	function onPressCallback () {
	}
	
	function onMouseMoveCallback(){
		
	}
	
	//convert a 2dim index to 1dim index
	function getOneDimIndex(){
		var cellIndex = getCellIndex();
		var itemsPerRow = listOwner.itemsPerRow;
		if(itemsPerRow  == undefined){
			//nearly all cases
			itemsPerRow = 1;
		}
/*		trace("cellIndex.itemIndex : "  + cellIndex.itemIndex);
		trace("cellIndex.columnIndex : "  + cellIndex.columnIndex);
		trace("indexAsNum : "  + indexAsNum);*/		
		return cellIndex.itemIndex * itemsPerRow + cellIndex.columnIndex;
		
		
	}
	function onRollOutCallback()
	{
		listOwner.dispatch( { type: "itemRollOut", index: getOneDimIndex(), target: listOwner});
	}
	function onRollOverCallback()
	{
		listOwner.dispatch( { type: "itemRollOver", index: getOneDimIndex(), target: listOwner});
	}
	function onReleaseCallback() {
		if(_isSelected){
			//already selected. do nothing
			return;
		}

		if(listOwner.multipleSelection){
			var indices = listOwner.selectedIndices;
			
//			trace("indices : "  + mx.data.binding.ObjectDumper.toString(indices));
			indices.push(getOneDimIndex());
			//explicitly call the setter function, otherwise its code won't be called
			listOwner.selectedIndices = indices;
		}else{
			listOwner.selectedIndex = getOneDimIndex();
		}
		listOwner.dispatch( { type: "onRelease", target: listOwner});
		trace("onRelease");
	}
	function onReleaseOutsideCallback() {
		trace("onReleaseOutsideCallback");
	}

	//called by parent after creation and setting of different callback elements 
	function setupWithListOwner(){
	}
	// note that setSize is implemented by UIComponent and calls size(), after setting
	// __width and __height
	
	function setValue(str:String, item:Object, sel:Boolean) : Void{

		trace(this  + "setValue "+str);
		for (var idx in item){
			trace(idx+"->"+item[idx]);
		}
		trace("selected : " + sel);
/**/
		
		//data 
		data = item.data;
		
		//icon must be here!
		this.icon = item.icon;
		trace("this.icon : " + this.icon);
		
		if (item == undefined){
			_visible = false;
			return;
		}
		// visible if there is data
		_visible = true;
		
		// selected item?
		selected_mc._visible = sel;
		
		_isSelected = sel;
		//trace("_isSelected" + _isSelected);
		doLater(this, "redraw");
	}

	function getPreferredHeight(Void) : Number{
		//trace("getPreferredHeight : " + _preferredHeight);
		return _preferredHeight;
	}

	function getPreferredWidth(Void) : Number{
		//trace("getPreferredWidth : " + _preferredWidth);
		return _preferredWidth;
	}
	function setSize(w,h) : Void{
		super.setSize(w,h);
		//trace("w:  " + w);
		//trace("h:  " + h);
		redraw();
	}
	/**
	 * redraw a cell: update position and size of the background clips
	 */
	function redraw()
	{
		selected_mc._width = bg_btn._width = width;
		selected_mc._height = bg_btn._height = height;
		selected_mc._x = selected_mc._y = bg_btn._x = bg_btn._y = 0;
	}
}