﻿/*
	this file is part of SILEX
	SILEX : RIA developement tool - see http://silex-ria.org/

	SILEX is (c) 2004-2007 Alexandre Hoyau and is released under the GPL License:

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License (GPL)
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/
/**
 * This class represents an application. It has the top level functions such as start, saveSection, openSection, ...
 * Handle the loading process of the app.
 * In the repository : /trunk/core/Application.as
 * @author	Alexandre Hoyau
 * @version	1
 * @date	2007-10-25
 * @mail : lex@silex.tv
 */
import org.silex.core.Utils;
import mx.events.EventDispatcher;
import org.silex.core.Constants;
import org.silex.core.Layout;
import org.silex.ui.Layer;
import org.silex.ui.menu.ContextMenuItemWithKeyboardShortcut;
import org.silex.ui.UiBase;
import org.silex.core.plugin.HookManager;
import org.silex.core.ApplicationHooks;

//////////////////////////////////
// Group: events
/////////////////////////////////
[Event("configLoaded")]
[Event("preloadDone")]
[Event("layoutLoaded")]
[Event("playersLoaded")] // thrown by org.silex.ui.Layer and get by org.silex.core.Deeplink and toolbox.Players
[Event("allPlayersLoaded")] // thrown by org.silex.ui.Layer 
[Event("saved")]
[Event("unsaved")]
[Event("resize")]
[Event("onBgPress")]
[Event("onBgRelease")]
[Event("openSection")] // sectionName, layoutFileName, target(i.e. Layout), deeplinkName


class org.silex.core.Application extends mx.core.UIObject{
	/**
	 * Reference to silex main Api object (org.silex.core.Api).
	 */
	private var silex_ptr:org.silex.core.Api;
	
	/**
	 * Used to store the stage position and size in _root coordinate system.
	 * Propeties: top,left,right,bottom.
	 */
	var stageRect:Object;
	/**
	 * Used to store the display region position and size in _root coordinate system.
	 * Propeties: top,left,right,bottom.
	 */
	var sceneRect:Object;
	
	/**
	 * Background clip.
	 */
	var bg_mc:MovieClip;
	
	/**
	 * Array of swf names.
	 * Each swf file should contain one font and will be loaded at start.
	 */
	var embeddedFonts:Array;

	//////////////////////////////////////
	// Group: files preload
	//////////////////////////////////////
	/**
	 * array of file names which will be preloaded, i.e. loaded before SILEX starts to load the 1st page of the website.
	 */
	private var preloadFilesList:Array;
	/**
	 * MovieClipLoader for the preloaded files.
	 */
	private var preload_mcl:MovieClipLoader;
	/**
	 * Container for the preloaded files.
	 */
	private var preload_mc:MovieClip;
	/**
	 * Index - in preloadFilesList, of the file which is curently being preloaded.
	 */
	var preloadIdx:Number=0;

	//////////////////////////////////////
	// Group: layouts
	//////////////////////////////////////
	/**
	 * Layout loader.
	 * Used to unload layouts (in org.silex.core.Layout) and to monitor loading.
	 */
	public var layout_mcl:MovieClipLoader;
	/**
	 * Listener object for the layout loader.
	 */
	var layoutListener:Object;
	/**
	 * Container for the first layout of the website.
	 */
	var layoutContainer:MovieClip;
	
	/**
	 * Layouts array: contains references to the layouts.
	 */
	var layouts:Array;
	
	/**
	 * Index of the interval used for the screen saver (setInterval ActionScript function).
	 */
	var screenSaverIntervalIdx:Number=-1;
	/**
	 * mouse listener for the screen saver activity detection
	 */
	var screenSaverMouseListener:Object;

	//////////////////////////////////////
	// Group: screen saver modes
	//////////////////////////////////////
	static var SCREEN_SAVER_MODE_ON:String="SCREEN_SAVER_MODE_ON";
	static var SCREEN_SAVER_MODE_OFF:String="SCREEN_SAVER_MODE_OFF";
	static var SCREEN_SAVER_MODE_ACTIVE:String = "SCREEN_SAVER_MODE_ACTIVE";
	/**
	 * screen saver current mode
	 */
	var screenSaverMode:String=SCREEN_SAVER_MODE_OFF;
	
	/**
	 * Constructor.
	 * Initialize context menu to display the about item.
	 */
	function Application(api:org.silex.core.Api){
		
		// store Api reference
		silex_ptr=api;
		silex_ptr.utils.trace("org.silex.core.Application constructor");

		//eventDispatcher
		EventDispatcher.initialize(this);
		
		// bg_mc init
		bg_mc=silex_ptr.parent.bg_mc;
		bg_mc.useHandCursor=false;
		bg_mc.onPress=Utils.createDelegate(this,bgPress);
		bg_mc.onRelease = Utils.createDelegate(this, bgRelease);
		
		// **
		// init preload of files
		//
		// init preload_mc
		if (!_root.preload_mc)
			_root.createEmptyMovieClip("preload_mc",_root.getNextHighestDepth());
		preload_mc=_root.preload_mc;
		preload_mc._visible=false;

		// listener for mcl
		var listener:Object=new Object;
		listener.onLoadInit=Utils.createDelegate(this,preloadFileSuccess);
		listener.onLoadError=Utils.createDelegate(this,preloadFileError);
		
		// init preload_mcl 
		preload_mcl=new MovieClipLoader;
		preload_mcl.addListener(listener);
		
		// **
		// init layout loading
		//
		//container for the first layout and section
		layoutContainer=silex_ptr.parent.createEmptyMovieClip("layoutContainer",silex_ptr.parent.getNextHighestDepth());

		// listener for mcl
		layoutListener=new Object;
		layoutListener.onLoadError=Utils.createDelegate(this,layoutLoadedError);
		layoutListener.onLoadComplete=Utils.createDelegate(this,layoutLoadedComplete);
		// in the layout		layoutListener.onLoadInit=Utils.createDelegate(this,layoutLoadedInit);
		
		// layout loader
		layout_mcl=new MovieClipLoader;
		layout_mcl.addListener(layoutListener);
	}
	//////////////////////////////////////
	// Group: background events
	//////////////////////////////////////
	function bgPress () {
		dispatchEvent({type:"onBgPress",target:this});
	}
	function bgRelease () {
		dispatchEvent({type:"onBgRelease",target:this});
	}

	function siteRefresh(){
		trace("siteRefresh");
		getURL("javascript:window.location.reload(false)");
	}
	function siteRefreshNoCache(){
		trace("siteRefreshNoCache");
		getURL("javascript:window.location.reload(true)");
	}
	/**
	 * Initialize context menu to display the about item.
	 */
	function initAfterConfigLoaded(){
		//silex_ptr.utils.trace("org.silex.core.Application initAfterConfigLoaded "+silex_ptr.config.CONTEXT_MENU_ABOUT_CAPTION);

		// get the language from user config
		if (silex_ptr.utils.isPartOf([System.capabilities.language],silex_ptr.config.AVAILABLE_LANGUAGE_LIST.split(","))){
			// silex_ptr.utils.trace("org.silex.core.Application initAfterConfigLoaded - language from user config is available : "+System.capabilities.language+" - "+silex_ptr.config.AVAILABLE_LANGUAGE_LIST);
			silex_ptr.config.language=System.capabilities.language;
		}
		else{
			if (silex_ptr.config.DEFAULT_LANGUAGE && silex_ptr.config.DEFAULT_LANGUAGE!=""){
				// silex_ptr.utils.trace("org.silex.core.Application initAfterConfigLoaded - language from user config is NOT available : "+System.capabilities.language+" - "+silex_ptr.config.AVAILABLE_LANGUAGE_LIST+" => "+silex_ptr.config.DEFAULT_LANGUAGE);
				silex_ptr.config.language=silex_ptr.config.DEFAULT_LANGUAGE;
			}
			else
			{
				// silex_ptr.utils.trace("org.silex.core.Application initAfterConfigLoaded - language from user config is NOT available AND THERE IS NO DEFAULT LANGUAGE => system language="+System.capabilities.language+" - languages list="+silex_ptr.config.AVAILABLE_LANGUAGE_LIST+" => language="+silex_ptr.config.language);
			}
		}
		// automatic language detection
		if (silex_ptr.config.ALLOW_AUTOMATIC_LANGUAGE_CHOICE!="false" && silex_ptr.utils.isPartOf([silex_ptr.config.language],silex_ptr.config.AVAILABLE_CONTEXT_LIST.split(",")))
		{
			if (!silex_ptr.config.globalContext)
				silex_ptr.config.globalContext = "";
			if (silex_ptr.config.globalContext != "")
				silex_ptr.config.globalContext += ",";
			silex_ptr.config.globalContext += silex_ptr.config.language;
		}

		// **
		// context menu
		// 
		if (!_root.menu) _root.menu=new ContextMenu();
		_root.menu.hideBuiltInItems();

		// about
		var aboutCaption_str:String=silex_ptr.utils.revealAccessors(silex_ptr.config.CONTEXT_MENU_ABOUT_CAPTION,silex_ptr.config);
		_root.menu.customItems.push(new ContextMenuItem(aboutCaption_str, Utils.createDelegate(this,aboutCallback)));

		// refresh
		var f5:Number = ContextMenuItemWithKeyboardShortcut.F5_ASCII;
		var caption:String = silex_ptr.config.CONTEXT_MENU_REFRESH_COMMAND;
/*creates problems on mac. disable till we can get some multisystem tests		
		_root.menu.customItems.push(new ContextMenuItemWithKeyboardShortcut(caption, Utils.createDelegate(this,siteRefresh), f5, false));
		caption = silex_ptr.config.CONTEXT_MENU_REFRESH_NO_CACHE_COMMAND;
		_root.menu.customItems.push(new ContextMenuItemWithKeyboardShortcut(caption, Utils.createDelegate(this,siteRefreshNoCache), f5, true));
*/		
		if (silex_ptr.config.ALLOW_LOGIN.toLowerCase()!="false"){
			// **
			// context menu
			// login/logout
			caption = silex_ptr.config.CONTEXT_MENU_LOGIN_CAPTION;
			_root.menu.customItems.push(new ContextMenuItemWithKeyboardShortcut(caption, Utils.createDelegate(this,loginCallback), "L", true));
			caption = silex_ptr.config.CONTEXT_MENU_LOGOUT_CAPTION;
			_root.menu.customItems.push(new ContextMenuItemWithKeyboardShortcut(caption, Utils.createDelegate(this,logoutCallback), "L", true));

			//flush logins 
			caption = silex_ptr.config.CONTEXT_MENU_FLUSH_LOGINS_CAPTION;
			_root.menu.customItems.push(new ContextMenuItem(caption, Utils.createDelegate(this,flushLoginsCallback)));
			
			// **
			// auto login
			var storedLoginInfo = SharedObject.getLocal(silex_ptr.config.SHARED_OBJECT_NAME);
			if(storedLoginInfo.data.isLoggedIn == true){
				loginCallback();
			}
		}
		refreshLoginMenuState();
		// **
		// stage, align and scale
		//
		// stage default values
		Stage.align="";
		Stage.scaleMode="noScale";

		// stage listener and resize
		var stageListener:Object = new Object();
		stageListener.onResize = Utils.createDelegate(this,stageResize);
		Stage.addListener(stageListener);
		// resize when a page has been opened (for Scroll mode)
		addEventListener("allPlayersLoaded", Utils.createDelegate(this, stageResize));
		// init the scene size
		stageResize();
		
		// **
		// bg color
		//
		if (silex_ptr.config.bgColor) setBgColor(silex_ptr.config.bgColor);
		
		// **
		// tracking
		//
		// google analytics
		var trackerInit_str:String="";
		if (silex_ptr.config.googleAnaliticsAccountNumber!="")
			trackerInit_str+="silexJsObj.setTrackingCode('"+silex_ptr.config.googleAnaliticsAccountNumber+"');";
		if (silex_ptr.config.phpmyvisitesURL!="")
			trackerInit_str+="silexJsObj.setPhpmyvisites('"+silex_ptr.config.phpmyvisitesURL+"','"+silex_ptr.config.phpmyvisitesSiteNumber+"');";

		if (trackerInit_str!=""){
			// tag current section
			trackerInit_str+="silexJsObj.tagCurrentSection();";
			// call javascript
			silex_ptr.com.jsCall(trackerInit_str);
			// silex_ptr.utils.trace("org.silex.core.Application initAfterConfigLoaded - tracker init "+trackerInit_str);
		}
		
		
		//  **
		// screen saver
		screenSaverInit();
		
		// **
		// mouse pointer
		silex_ptr.constants.mouse=(silex_ptr.config.mouse!="false");
	}
	/**
	 * Callback executed when the window is resized
	 */
	function stageResize() {
		// **
		// compute stage position and height in _root coordinate system
		// 
		stageRect=new Object;
		// X axis
		var stageCenterX:Number=silex_ptr.config.LOADER_STAGE_HALF_SIZE_X; // silex.fla stage size
		var stageHalfWidth=Stage.width/2;
		stageRect.left=stageCenterX-stageHalfWidth;
		stageRect.right=stageCenterX+stageHalfWidth;
		// Y axis
		var stageCenterY:Number=silex_ptr.config.LOADER_STAGE_HALF_SIZE_Y;
		var stageHalfHeight=Stage.height/2;
		stageRect.top=stageCenterY-stageHalfHeight;
		stageRect.bottom=stageCenterY+stageHalfHeight;

		//silex_ptr.utils.trace("org.silex.core.Application stageResize stageRect="+stageRect.left+","+stageRect.top+" - "+stageRect.right+","+stageRect.bottom);

		// **
		// position of background and tools
		// 
		bg_mc._x=stageRect.left;
		bg_mc._width=Stage.width;
		bg_mc._y=stageRect.top;
		bg_mc._height=Stage.height;
		
		//silex_ptr.utils.trace("org.silex.core.Application stageResize "+silex_ptr.config.scaleMode+"("+bg_mc._x+","+bg_mc._width+","+silex_ptr.config.LOADER_STAGE_HALF_SIZE_Y+","+silex_ptr.config.LOADER_STAGE_HALF_SIZE_X+")");

		// **
		// scene alignment in Stage
		// lower case (case insensitive)
		var scaleModeLowerCase:String = silex_ptr.config.scaleMode.toLowerCase();
		
		// pixel mode case: showAll if stage is smaller than scene, and noScale otherwise
		if (scaleModeLowerCase == Constants.SCALE_MODE_PIXEL)
			if (Stage.width >= silex_ptr.config.layoutStageWidth && Stage.height >= silex_ptr.config.layoutStageHeight)
				scaleModeLowerCase = Constants.SCALE_MODE_NO_SCALE;
			else
				scaleModeLowerCase = Constants.SCALE_MODE_SHOW_ALL;
		
		// change size and placement
		switch (scaleModeLowerCase){
			case Constants.SCALE_MODE_SCROLL:
				silex_ptr.sequencer.addItem(null,Utils.createDelegate(this,updateSceneSizeAfterPagesAreClosed));
			break;
			case Constants.SCALE_MODE_NO_SCALE:
				layoutContainer._xscale=100;
				layoutContainer._yscale=100;
				centerScene();
			break;
			default:
			case Constants.SCALE_MODE_SHOW_ALL:
				stretchScene();
			break;
		}
		// dispatchEvent
		dispatchEvent( { type:"resize", target:this } );
	}
	function updateSceneSizeAfterPagesAreClosed() {
		layoutContainer._xscale=100;
		layoutContainer._yscale=100;
		centerScene(false);
		
		// resize in function of the content height
		var maxHeight:Number = getMaxContentHeight();
		
		// resize scene rect
		sceneRect.top=stageRect.top;
		sceneRect.bottom = sceneRect.top + maxHeight;
		
		// dispatch to javascript
		var jsCommand_str:String="adjustSilexWindowSize('"+maxHeight+"');";

		// call javascript
		if (silex_ptr.config.ENABLE_DEEPLINKING!="false")
			silex_ptr.com.jsCall(jsCommand_str);
	}
	/**
	 * compute the content maximum height in all opened pages
	 */
	function getMaxContentHeight() {
		// silex_ptr.utils.trace("org.silex.core.Application getMaxContentHeight ");
		var maxHeight:Number = silex_ptr.config.layoutStageHeight;
		// browse all layouts
		for (var idxLayout:Number = 0; idxLayout < layouts.length; idxLayout++) {
			var layout:Layout = layouts[idxLayout];
			// browse all layers
			for (var idxLayer:Number = 0; idxLayer < layout.layers.length; idxLayer++) {
				var layer:Layer = layout.layers[idxLayer];
				// browse all players
				for (var idxPlayer:Number = 0; idxPlayer < layer.players.length; idxPlayer++) {
					var playerInfo:Object = layer.players[idxPlayer];
					var player:UiBase = layer[layer.players[idxPlayer]._name].main;
					var playerBottom:Number = player.height + player._y;
					if (maxHeight < playerBottom){
						maxHeight = playerBottom;
					}
				}
			}
		}
		return maxHeight;
	}
	/**
	 * Center layouts and content <=> compute offset to apply to the scene.
	 * layoutStageWidth and layoutStageHeight are read in config file, default is 800x600.
	 */
	function stretchScene() {
		var sceneWidth:Number=800;
		var sceneHeight:Number=600;
		if (silex_ptr.config.layoutStageWidth) sceneWidth=parseInt(silex_ptr.config.layoutStageWidth);
		if (silex_ptr.config.layoutStageHeight) sceneHeight=parseInt(silex_ptr.config.layoutStageHeight);

		// compute scale to apply to the scene
		var sceneScaleX:Number=Stage.width/sceneWidth;
		var sceneScaleY:Number=Stage.height/sceneHeight;
		
		// apply scale
		var sceneScaleFactor:Number=Math.round(100*(Math.min(sceneScaleX,sceneScaleY)));
		layoutContainer._xscale=layoutContainer._yscale=sceneScaleFactor;
		
		// center
		var newSceneWidth=sceneWidth*sceneScaleFactor/100;
		var newSceneHeight=sceneHeight*sceneScaleFactor/100;
		var sceneCenterX:Number=newSceneWidth/2;
		var sceneCenterY:Number=newSceneHeight/2;
		var stageCenterX:Number=silex_ptr.config.LOADER_STAGE_HALF_SIZE_X;
		var stageCenterY:Number=silex_ptr.config.LOADER_STAGE_HALF_SIZE_Y;
		var offsetX:Number=stageCenterX-sceneCenterX;
		var offsetY:Number=stageCenterY-sceneCenterY;
		
		layoutContainer._x=offsetX;
		layoutContainer._y=offsetY;

		//silex_ptr.utils.trace("org.silex.core.Application stretchScene "+sceneScaleX+","+sceneScaleY);
		
		// **
		// compute sceneRect
		// 
		sceneRect=new Object;
		sceneRect.left=stageCenterX-Math.round(newSceneWidth/2);
		sceneRect.top=stageCenterY-Math.round(newSceneHeight/2);;
		sceneRect.right=stageCenterX+Math.round(newSceneWidth/2);
		sceneRect.bottom=stageCenterY+Math.round(newSceneHeight/2);;;
	}
	/**
	 * Center layouts and content <=> compute offset to apply to the scene.
	 * layoutStageWidth and layoutStageHeight are read in config file, default is 800x600.
	 */
	function centerScene(centerVertically:Boolean, centerHorizontally:Boolean) {
		// default values
		if (centerHorizontally == undefined) centerHorizontally = true;
		if (centerVertically == undefined) centerVertically = true;
		
		// determine scene size
		var sceneWidth:Number=800;
		var sceneHeight:Number=600;
		if (silex_ptr.config.layoutStageWidth) sceneWidth=parseInt(silex_ptr.config.layoutStageWidth);
		if (silex_ptr.config.layoutStageHeight) sceneHeight=parseInt(silex_ptr.config.layoutStageHeight);

		// sceneCenter - stageCenter = offset vector (offset to apply to the scene)
		var sceneCenterX:Number=Math.round(sceneWidth/2);
		var sceneCenterY:Number=Math.round(sceneHeight/2);
		var stageCenterX:Number=silex_ptr.config.LOADER_STAGE_HALF_SIZE_X;
		var stageCenterY:Number=silex_ptr.config.LOADER_STAGE_HALF_SIZE_Y;
		var offsetX=stageCenterX-sceneCenterX;
		var offsetY=stageCenterY-sceneCenterY;
		
		//silex_ptr.utils.trace("org.silex.core.Application centerScene "+offsetX+","+offsetY);

		// **
		// compute sceneRect and apply offset
		// 
		sceneRect = new Object;
		if (centerHorizontally)
		{
			// center
			layoutContainer._x=offsetX;
			sceneRect.left=stageCenterX-sceneCenterX;
			sceneRect.right=stageCenterX+sceneCenterX;
		}
		else
		{
			// align left
			layoutContainer._x = stageRect.left;
		}
		if (centerVertically)
		{
			// center
			layoutContainer._y=offsetY;
			sceneRect.top=stageCenterY-sceneCenterY;
			sceneRect.bottom = stageCenterY + sceneCenterY;
		}
		else
		{
			// align top
			layoutContainer._y=stageRect.top;
		}
	}
	/**
	 * Change bg_mc color.
	 * @param	color_str	the color to apply
	 * @example	FFFFFF for white
	 */
	function setBgColor(color_str:String){
		var _color:Color=new Color(bg_mc);
		_color.setRGB(parseInt(color_str,16));
		
		// change html page color
		//silex_ptr.com.jsCall("void(document.bgColor='#"+color_str+"')");
	}
	/**
	 * Callback function executed when the user select "about SILEX" in the context menu.
	 * Execute the command CONTEXT_MENU_ABOUT_COMMAND read in the config files
	 */
	function aboutCallback(){
		// silex_ptr.utils.trace("org.silex.core.Application aboutCallback "+silex_ptr.config.CONTEXT_MENU_ABOUT_COMMAND);
		silex_ptr.interpreter.exec(silex_ptr.config.CONTEXT_MENU_ABOUT_COMMAND);
	}
	/**
	 * Build the context menu for SILEX.
	 * Login, logout and about menus.
	 */
	function refreshLoginMenuState(){
//		trace("refreshLoginMenuState  :" + mx.data.binding.ObjectDumper.toString(_root.menu.customItems));
		var isLoggedIn:Boolean = silex_ptr.authentication.isLoggedIn();

		var storedLoginInfo = SharedObject.getLocal(silex_ptr.config.SHARED_OBJECT_NAME);
		//trace("storedLoginInfo  :" + mx.data.binding.ObjectDumper.toString(storedLoginInfo));
		var enableFlushLogins:Boolean = false;
		if(storedLoginInfo.data.loginArray != null){
			enableFlushLogins = true;
		}
		
		var len = _root.menu.customItems.length;
		for(var i = 0; i < len; i++){
			var item = _root.menu.customItems[i];
			if(item.caption.indexOf("login ") != -1) //the " " is important because otherwise we catch "flush logins"
				item.enabled = !isLoggedIn;
			if(item.caption.indexOf("logout") != -1)
				item.enabled = isLoggedIn;
			if(item.caption.indexOf("flush logins") != -1)
				item.enabled = enableFlushLogins;
		}
	}
	/**
	 * Callback function executed when the user select "login" in the context menu.
	 * Check if the user has  a login + password stored in the cookie and eventually open a login dialog.
	 * @see	org.silex.core.Utils::promptPassword
	 */
	function loginCallback(){
		//silex_ptr.utils.trace("org.silex.core.Application loginCallback ");
		//look for stored password and login directly if found. else prompt for password
		var loginInfoFound:Boolean = false;
		var storedLoginInfo = SharedObject.getLocal(silex_ptr.config.SHARED_OBJECT_NAME);
		//this.silex_ptr.utils.trace("org.silex.core.Application stored info  : " + mx.data.binding.ObjectDumper.toString(storedLoginInfo.data));
		if(storedLoginInfo.data.loginArray){
			for(var i = 0; i < storedLoginInfo.data.loginArray.length; i++){		
				var info = storedLoginInfo.data.loginArray[i];
				if(info.url == _level0._url){
					trace("logging in : " + info.login_str + ", " + info.pass_str);
					silex_ptr.authentication.login(info.login_str, info.pass_str,Utils.createDelegate(this,authenticationCallback));
					loginInfoFound = true;
					break;
				}
			}
		}
		if(!loginInfoFound)		
			silex_ptr.utils.promptPassword(silex_ptr.config.PROMPT_LOGIN_PASS,Utils.createDelegate(this,promptPasswordCallback));
	}
	/**
	 * Callback function executed when the user select "flush login" in the context menu.
	 */
	function flushLoginsCallback() {
		silex_ptr.utils.trace("org.silex.core.Application flushLoginsCallback ");
		var storedLoginInfo:SharedObject = SharedObject.getLocal(silex_ptr.config.SHARED_OBJECT_NAME);
//		var _str:String = "x---x";
/*		for (var prop:String in storedLoginInfo.data)
		{
			storedLoginInfo.data[prop] = undefined;
//			_str += prop + "-";
		}
//		_root.getURL(_str, "_blank");
//		storedLoginInfo.data.loginArray = null;
//		storedLoginInfo.data.toolsPostionsArray = new Array;
//		storedLoginInfo.data = new Object;
		storedLoginInfo.flush();
*/		storedLoginInfo.clear();
		storedLoginInfo.flush();
		refreshLoginMenuState();

		if (silex_ptr.authentication.isLoggedIn)
		{
			storedLoginInfo = SharedObject.getLocal(silex_ptr.config.SHARED_OBJECT_NAME+"-"+silex_ptr.authentication.currentLogin);
			storedLoginInfo.clear();
			storedLoginInfo.flush();
		}
	}
	/**
	 * Callback function executed when the user select "login" in the context menu and do not have a login + password stored in the cookie.
	 * Called by the promptPassword dialog function opened by the loginCallback function.
	 * @see	org.silex.core.Application::loginCallback
	 * @see	org.silex.core.Utils::promptPassword
	 * @param	login_str	the user login
	 * @param	pass_str	the user password
	 */
	function promptPasswordCallback(login_str,pass_str) {
		// silex_ptr.utils.trace("org.silex.core.Application promptPasswordCallback ");
		silex_ptr.authentication.login(login_str,pass_str,Utils.createDelegate(this,authenticationCallback));
		// display message
		silex_ptr.utils.alertSimple(silex_ptr.utils.revealAccessors(silex_ptr.config.MESSAGE_PLEASE_WAIT_WHILE_CHECKING_LOGIN_PASS,{login:login_str,pass:pass_str}),1500);
	}
	/**
	 * Callback function executed when the result of the authentication is arrived.
	 * @param	success	the state of the authentication. true if it succeded and false otherwise.
	 */
	function authenticationCallback(success:Boolean) {
		// silex_ptr.utils.trace("org.silex.core.Application authenticationCallback ");
		silex_ptr.toolsManager.showToolboxes(success);
		silex_ptr.config.globalContext = silex_ptr.config.AVAILABLE_CONTEXT_LIST;
		refreshLoginMenuState();
	}
	/**
	 * Callback function executed when the user select "logout" in the context menu.
	 */
	function logoutCallback(){
		// silex_ptr.utils.trace("org.silex.core.Application logoutCallback ");
		silex_ptr.authentication.logout();
		silex_ptr.toolsManager.showToolboxes(false);
		//setEditable(false)  for each player
		for (var idxLayout:Number=0;idxLayout<layouts.length;idxLayout++){
			for (var idxLayer:Number=0;idxLayer<layouts[idxLayout].layers.length;idxLayer++)
				layouts[idxLayout].layers[idxLayer].setEditable(false);
		}
	}

	/**
	 * Start the loading process.
	 * Call com.getDynData.
	 * All _root properties of type number or string are considered as params passed to Flash by javascript.
	 */
	function start() {
		
		// layouts array
		layouts=new Array;
		
		// bgColor
		silex_ptr.parent.bgColor_mc._visible=false;

		// silex_ptr.utils.trace("org.silex.core.Application start - variables passed to flash:");
		// store all strings and numbers into the websitInfo object
		var websiteInfo:Object=new Object;
		for (var propName_str:String in _root){
			// is it of type number or string
			if (typeof(_root[propName_str])=="string" || typeof(_root[propName_str])=="number"){
				websiteInfo[propName_str]=_root[propName_str];
			}
		}
		
		// get the admin language from website config
		if (silex_ptr.utils.isPartOf([System.capabilities.language],_root.SILEX_ADMIN_AVAILABLE_LANGUAGES.split(","))){
			// silex_ptr.utils.trace("org.silex.core.Application start - language from website config is available : "+System.capabilities.language+" - "+_root.SILEX_ADMIN_AVAILABLE_LANGUAGES);
			silex_ptr.config.adminLanguage=System.capabilities.language;
		}
		else{
			// silex_ptr.utils.trace("org.silex.core.Application start - language from website config is NOT available : "+System.capabilities.language+" - "+_root.SILEX_ADMIN_AVAILABLE_LANGUAGES+" => "+_root.SILEX_ADMIN_DEFAULT_LANGUAGE);
			silex_ptr.config.adminLanguage=_root.SILEX_ADMIN_DEFAULT_LANGUAGE;
		}

		// extract files list
		// remove blank spaces before and after ","
		_root.config_files_list=silex_ptr.utils.replace(_root.config_files_list,", ",",");
		_root.config_files_list=silex_ptr.utils.replace(_root.config_files_list," ,",",");
		// split the list
		var filesList:Array;

		// init conf files list
		if (_root.config_files_list && _root.config_files_list!="")
			filesList = _root.config_files_list.split(",");
		else
			filesList = new Array;

		// adds language file
		if (_root.forceLanguageFile)
		{
			if (_root.forceLanguageFile != "" && _root.forceLanguageFile != "none")
			{
				// add the desired language file
				filesList.push(_root.forceLanguageFile);
			}
			else
			{
				// do not add any language file (cd-r case)
			}
		}
		else
		{
			// add the appropriate language file
			filesList.push(silex_ptr.utils.revealAccessors(silex_ptr.config.languageConfigFileName,silex_ptr.config));
		}
		
		silex_ptr.utils.trace("org.silex.core.Application start - config_files_list="+filesList);

		/*
		// insert website conf file
		if (filesList.toString().indexOf(silex_ptr.constants.initialContentFolderPath+websiteInfo.id_site+"/"+silex_ptr.constants.CONFIG_WEBSITE_CONF_FILE)==-1){
			this.silex_ptr.utils.trace("org.silex.core.Application start - " + filesList.toString()+" do not contain the conf file ("+silex_ptr.constants.initialContentFolderPath+websiteInfo.id_site+"/"+silex_ptr.constants.CONFIG_WEBSITE_CONF_FILE+")");
			filesList.push(silex_ptr.constants.initialContentFolderPath+websiteInfo.id_site+"/"+silex_ptr.constants.CONFIG_WEBSITE_CONF_FILE);
		}
		*/
		
		// call getDynData to load config files
		silex_ptr.com.getDynData(websiteInfo,filesList,Utils.createDelegate(this,dynDataLoaded));
	}
	/**
	 * Override constants and config variables.
	 * Init the Constant class with these constants.
	 * Continue the loading process (preload files).
	 */
	function dynDataLoaded(result:Array) {
		// silex_ptr.utils.trace("org.silex.core.Application dynDataLoaded "+result.length+" - "+silex_ptr.config.preload_files_list);
		var isConfFileFound:Boolean=false;
		
		if (result)
		{
			// **
			// overrides constants and config
			// 
			isConfFileFound=overrideConfig(result);
			// if config file is not found
			// - if we are in widget mode, the config was loaded from widget.php
			// - otherwise the website does not exist => refresh for the php to handle the 404 page
//			if (isConfFileFound == false && silex_ptr.config.widgetMode != "true")
			if (isConfFileFound == false)
			{
				// call getDynData to load config files
				openWebsite(silex_ptr.config.id_site);
/*				if (layouts[0])
					layouts[0].close();
				silex_ptr.sequencer.addItem(null,Utils.createDelegate(this,siteRefresh));
*/
				// -------------------
				return;
				// -------------------
				
				// This website does not exist yet
				var _mc:MovieClip = this.silex_ptr.utils.alert(silex_ptr.utils.revealAccessors(silex_ptr.config.MESSAGE_WEBSITE_DOES_NOT_EXIST, silex_ptr.config));
				
				// put window in 0,0 since the size of the scene is about to be changed (in function of the data read in conf files)
				_mc._x=_mc._y=0;
			}
			// **
			// init app
			// after config has been loaded
			//silex_ptr.sequencer.doInNFrames(20,Utils.createDelegate(this,initAfterConfigLoaded));
			initAfterConfigLoaded();

			// dispatch event
			dispatchEvent({type:"configLoaded",target:this});

			// **
			// preload files
			// 
			// fill the the preloadFilesList array
			var preloadFilesListLocal:Array;
			if (silex_ptr.config.preload_files_list && silex_ptr.config.preload_files_list != ""){
				// silex_ptr.utils.trace("org.silex.core.Application dynDataLoaded - preload_files_list exist");
				// remove blank spaces before and after ","
				silex_ptr.config.preload_files_list=silex_ptr.utils.replace(silex_ptr.config.preload_files_list,", ",",");
				silex_ptr.config.preload_files_list=silex_ptr.utils.replace(silex_ptr.config.preload_files_list," ,",",");
				silex_ptr.config.preload_files_list=silex_ptr.utils.replace(silex_ptr.config.preload_files_list,"\r",",");
				// split the list
				preloadFilesListLocal=silex_ptr.config.preload_files_list.split(",");
			}
			// silex_ptr.utils.trace("org.silex.core.Application dynDataLoaded - preloadFilesList: "+preloadFilesListLocal);

			//silex_ptr.sequencer.doInNFrames(20,Utils.createDelegate(this,preloadFilesStart));
			preloadFilesStart(preloadFilesListLocal);
		}
		else{
			// silex_ptr.utils.trace("org.silex.core.Application dynDataLoaded - error loading conf files ("+preloadFilesList+")",silex_ptr.constants.DEBUG_LEVEL_ERROR);
			silex_ptr.utils.alert("error loading conf files ("+result.toString()+")");
		}
	}
	/** 
	 * Override constants and config variables.
	 */
	function overrideConfig(result:Array):Boolean{
		var isConfFileFound:Boolean=false;
		// enumarate each file
		for (var idx:Number=0;idx<result.length;idx++){
			// silex_ptr.utils.trace("org.silex.core.Application overrideConfig - the config file #"+idx+" has "+result[idx].length+" lines");
			// enumarate each line of the file
			for (var line_num:Number=0;line_num<result[idx].length;line_num++){
				// if there is a variable and its value and no ";" as first char
				if (result[idx][line_num].charAt(0)!=";" && result[idx][line_num].indexOf("=",1)>0){
					var _lv:LoadVars=new LoadVars;
					_lv.onData(result[idx][line_num]);

					// copy each variable into config object
					for (var propName_str:String in _lv){
						if (propName_str!="" && propName_str!="\n" && propName_str!="\r" && propName_str!="loaded"){
							//silex_ptr.utils.trace("org.silex.core.Application overrideConfig - config variable: "+propName_str+" = "+_lv[propName_str]);
							silex_ptr.config[propName_str]=_lv[propName_str];
							
							if (propName_str=="layerSkinUrl"){
								// if the conf.txt file was loaded, then layerSkinUrl is set (at least)
								isConfFileFound=true;
							}
						}
					}
				}
			}				
		}
		return isConfFileFound;
	}
	/**
	 * Callback executed when a file has been successfully preloaded.
	 * Preload the next file.
	 */
	function preloadFileSuccess(){
	    // silex_ptr.utils.trace("org.silex.core.Application preloadFileSuccess");
		silex_ptr.sequencer.doInNextFrame(Utils.createDelegate(this,preloadNextFile));
	}
	/**
	 * Callback executed when a file has *not* been successfully preloaded.
	 * Preload the next file.
	 */
	function preloadFileError(){//target_mc:MovieClip, errorCode:String, httpStatus:Number) {
	    silex_ptr.utils.trace("org.silex.core.Application preloadFileError>> loadListener.onLoadError()");
	    // silex_ptr.utils.trace("org.silex.core.Application preloadFileError>> ==========================");
//	    silex_ptr.utils.trace("org.silex.core.Application preloadFileError>> errorCode: " + errorCode);
//	    silex_ptr.utils.trace("org.silex.core.Application preloadFileError>> httpStatus: " + httpStatus);
		silex_ptr.sequencer.doInNextFrame(Utils.createDelegate(this,preloadNextFile));
	}
	/**
	 * Preloads the next file in preloadFilesList array.
	 * Or call preloadFilesDone if the end of the array was reached.
	 */
	function preloadNextFile()
	{
		// silex_ptr.utils.trace("org.silex.core.Application preloadNextFile "+preloadFilesList);
		// is there a file to preload?
		if (preloadFilesList && preloadFilesList.length > 0)
		{
			// remove previous clip
			preload_mcl.unloadClip(preload_mc["mc"+preloadIdx.toString()]);
			
			// start loading next clip
			preloadIdx++;
			
			// create a movie clip
			var newPreload_mc:MovieClip = preload_mc.createEmptyMovieClip("mc"+preloadIdx.toString(),preload_mc.getNextHighestDepth());
			
			// preload the file
			// silex_ptr.utils.trace("org.silex.core.Application preloadNextFile ("+preloadIdx.toString()+") - list = "+preloadFilesList);
			preload_mcl.loadClip(String(silex_ptr.rootUrl+preloadFilesList.pop()),newPreload_mc);
		}
		else{
			// empty the preload mc
			//preload_mcl.unloadClip(preload_mc);
			// call preloadFilesDone
			preloadFilesDone();
		}
	}
	/**
	 * preload files start<br />
	 * call hooks<br />
	 * call preloadNextFile
	 */
	function preloadFilesStart(preloadFilesListLocal:Array) {
		// call hooks
        var hookManager:HookManager = HookManager.getInstance();
		hookManager.callHooks(ApplicationHooks.PRELOAD_START_HOOK_UID, arguments);
		// store the files list
		preloadFilesList = preloadFilesListLocal;
		// start preloading the first file
		preloadNextFile();
	}
	/**
	 * Callback executed when all files have been loadded, with errors or not.
	 * call hooks<br />
	 */
	function preloadFilesDone() {
		silex_ptr.utils.trace("org.silex.core.Application preloadFilesDone - all files have been loaded");
		// call hooks
        var hookManager:HookManager = HookManager.getInstance();
		hookManager.callHooks(ApplicationHooks.PRELOAD_END_HOOK_UID, arguments);
		
		// dispatch event
		dispatchEvent({type:"preloadDone",target:this});

		// open the CONFIG_START_SECTION xml file in the initialLaytouFile layout file
		openSection(silex_ptr.config.CONFIG_START_SECTION,silex_ptr.config.initialLayoutFile,null,silex_ptr.config.CONFIG_START_SECTION);
		
		// initialize deeplinks
//		if (silex_ptr.config.initialPath)
//			silex_ptr.deeplink.currentPath=silex_ptr.config.initialPath;
	}
	/** 
	 * Close current website and open an other website on the same server.
	 */
	function openWebsite(idsite:String){
		if (layouts[0])
			layouts[0].close();
		/*silex_ptr.config.id_site=*///_root.id_site=idsite;
/*		silex_ptr.deeplink.changeWebsite(idsite);
		silex_ptr.sequencer.addItem(null,Utils.createDelegate(this,siteRefresh));*/
		getURL("javascript:window.location="+silex_ptr.rootUrl+"?/"+idsite);
	}
	/**
	 * Start the loading process of a section.<br />
	 * call hooks<br />
	 * @param sectionName_str	section name that will give us the name of the xml file containing the data
	 * @param layoutFile		layout swf file (does not contain the path to the layout folder of the website)
	 * @param targetLayout		org.silex.core.Layout object in which to open the section (load the layout)
	 * @return					true if the section loading could be started
	 */
	function openSection(sectionName_str:String,layoutFileName_str:String,targetLayout:org.silex.core.Layout,deeplinkName:String):Boolean{
		// call hooks
        var hookManager:HookManager = HookManager.getInstance();
		hookManager.callHooks(ApplicationHooks.OPENSECTION_START_HOOK_UID, arguments);
		
		silex_ptr.utils.trace("org.silex.core.Application openSection "+sectionName_str+", "+layoutFileName_str+", "+targetLayout+", "+deeplinkName);
		
		// check inputs
		if (!sectionName_str || !layoutFileName_str || !deeplinkName || sectionName_str=="" || layoutFileName_str==""){
			silex_ptr.utils.trace("org.silex.core.Application openSection error in the input: "+sectionName_str+", "+layoutFileName_str+", "+deeplinkName,silex_ptr.constants.DEBUG_LEVEL_ERROR);
			return false;
		}
		
		// build the layout file name
		var layoutFileUrl_str:String=silex_ptr.config.layoutFolderPath+layoutFileName_str;
		
		// store section name in the listener
		layoutListener.sectionName=sectionName_str;
		layoutListener.deeplinkName=silex_ptr.utils.cleanID(deeplinkName);
		layoutListener.layoutFileName=layoutFileName_str;
		
		// get the layer's container and load the next layout
		if (!targetLayout){
			// case of the first layout: open in the application layout container
			// hide  while loading
			//layoutContainer._visible=false;
			// start loading the layout
			layout_mcl.loadClip(silex_ptr.rootUrl+layoutFileUrl_str,layoutContainer);
		}
		else{
			var targetClip:MovieClip;
			// create a layout container if needed
			if (!targetLayout.layoutContainer) 
				targetLayout.layoutContainer = targetLayout._parent.createEmptyMovieClip("layoutContainer", targetLayout._parent.getNextHighestDepth());
				
			// use the layout's layout container 
			targetClip = targetLayout.layoutContainer;

			// close current child layout
			if (targetLayout.currentChildLayoutContainer)
			{
				// is the layout loaded or is it still loading?
				if (targetLayout.currentChildLayoutContainer.silexLayout)
					targetLayout.currentChildLayoutContainer.silexLayout.close();
				else{
					// could not open section because a layout is still loading
					silex_ptr.utils.trace("org.silex.core.Application openSection error: could not open section because a layout is still loading - "+sectionName_str+", "+layoutFileName_str,silex_ptr.constants.DEBUG_LEVEL_WARNING);
					return false; 
				}
					// bug in the sequencer: layout_mcl.unloadClip(targetLayout.currentChildLayoutContainer);
			}
			else
			{
				// for the first child layout only
				
				// start transition
				silex_ptr.sequencer.addItem(targetLayout.anims[silex_ptr.config.ANIM_NAME_TRANSITION]);
				
				// event
				targetLayout.dispatchEvent({type:"showChild",deeplinkName:silex_ptr.utils.cleanID(deeplinkName),cleanSectionName:silex_ptr.utils.cleanID(sectionName_str),sectionName:sectionName_str,layoutFileName:layoutFileName_str,target:targetLayout});
			}
			
			// create a new container
			targetLayout.currentChildLayoutContainer=targetClip.createEmptyMovieClip("layout"+targetLayout.currentChildLayoutContainerIndex++,targetClip.getNextHighestDepth());
			// hide  while loading
			//targetLayout.currentChildLayoutContainer._visible=false;

			// start loading the layout
			layout_mcl.loadClip(silex_ptr.rootUrl+layoutFileUrl_str,targetLayout.currentChildLayoutContainer);
		}
		// event
		dispatchEvent({type:"openSection", sectionName:sectionName_str, deeplinkName:silex_ptr.utils.cleanID(deeplinkName),cleanSectionName:silex_ptr.utils.cleanID(sectionName_str),layoutFileName:layoutFileName_str,target:targetLayout});
		//targetLayout.dispatchEvent({type:"showChild",sectionName:sectionName_str,layoutFileName:layoutFileName_str,target:targetLayout});
		
		// 
		hookManager.callHooks(ApplicationHooks.OPENSECTION_END_HOOK_UID, null);

		
		// 
		return true;
	}
	/**
	 * Called by layout_mcl when a layout is not loaded correctly.
	 * handle error
	 * @param target_mc		movie clip in which the section (the layout) was supposed to be loaded
	 * @param errorCode		A string that explains the reason for the failure, either "URLNotFound" or "LoadNeverCompleted".
	 */
	function layoutLoadedError(target_mc:MovieClip, errorCode:String){
		// handle error
		silex_ptr.utils.trace("org.silex.core.Application layoutLoadedError error loading layout "+layoutListener.layoutFileName,silex_ptr.constants.DEBUG_LEVEL_ERROR);
		silex_ptr.utils.alert(silex_ptr.utils.revealAccessors(silex_ptr.config.ERROR_LOADING_LAYOUT,{layoutFileName:layoutListener.layoutFileName,error:errorCode}));
		
		// ?? to remove ??
		// layoutLoadedComplete(target_mc);
		// layoutLoadedInit(target_mc);
	}
	/**
	 * Called by layout_mcl when a layout is loaded BEFORE 1st frame.
	 * create silexLayout object at the root of this layout and starts the xml loading process
	 * @param target_mc	movie clip in which the section (the layout) is loaded
	 */
	function layoutLoadedComplete(target_mc:MovieClip){
		silex_ptr.utils.trace("org.silex.core.Application layoutLoadedComplete "+target_mc+" - "+layoutListener.sectionName+","+layoutListener.layoutFileName+","+layoutListener.deeplinkName);
		
		// workaround bug when layoutLoadedInit is called before layoutLoadedComplete because an other layout was preloaded
		layoutListener.target=target_mc;
		
		// create silexLayout object
		target_mc.silexLayout = new org.silex.core.Layout(silex_ptr, target_mc, layoutListener.sectionName, layoutListener.layoutFileName, layoutListener.deeplinkName);
		
		silex_ptr.sequencer.doInNFrames(1,Utils.createDelegate(this, layoutLoadedInitNew),target_mc);
	}

	/**
	 * Called by layout_mcl when a layout is loaded AFTER 1st frame.
	 * @param target_mc	movie clip in which the section (the layout) is loaded
	 */
	function layoutLoadedInit(target_mc:MovieClip) {
		silex_ptr.utils.trace("org.silex.core.Application layoutLoadedInit "+target_mc._target);
	}
	function layoutLoadedInitNew(target_mc:MovieClip) {
		// call hooks
        var hookManager:HookManager = HookManager.getInstance();
		hookManager.callHooks(ApplicationHooks.LAYOUTINIT_START_HOOK_UID, arguments);

		silex_ptr.utils.trace("org.silex.core.Application layoutLoadedInitNew "+target_mc._target);
		
		// workaround bug when layoutLoadedInit is called before layoutLoadedComplete because an other layout was preloaded
		if (layoutListener.target != target_mc)
		{
			silex_ptr.utils.trace("org.silex.core.Application layoutLoadedInit error: workaround bug when layoutLoadedInit is called before layoutLoadedComplete because an other layout was preloaded");
			//silex_ptr.sequencer.doInNextFrame(Utils.createDelegate(this, layoutLoadedInit),target_mc);
			return;
		}

		// initialize the layout created in layoutLoadedComplete
		target_mc.silexLayout.init();
		
		// workaround bug when layoutLoadedInit is called before layoutLoadedComplete because an other layout was preloaded
		// layoutListener.sectionName=undefined;
		// layoutListener.layoutFileName = undefined;
		layoutListener.target = undefined;
		
		registerLayout(target_mc.silexLayout);
		
		// show after loading
		//target_mc._visible=true;

		// dispatch event
		dispatchEvent( { type:"layoutLoaded", target:target_mc.silexLayout } );
		// hooks
		hookManager.callHooks(ApplicationHooks.LAYOUTINIT_END_HOOK_UID, null);
	}
	/**
	 * Called by layoutLoadedInit.
	 * Add the layout to layouts array and set its onUnload function.
	 * @param silexLayout	the layout to register
	 */
	function registerLayout(silexLayout:org.silex.core.Layout) {
		silex_ptr.utils.trace("org.silex.core.Application registerLayout - silexLayout="+silexLayout);
		layouts.push(silexLayout);
		silexLayout._parent.onUnload=function(){
			_global.getSilex(this).utils.trace("org.silex.core.Application registerLayout silexLayout.onUnload - this="+this+" - this.silexLayout="+this.silexLayout);
			_global.getSilex(this).application.unRegisterLayout(this.silexLayout);
		}
	}
	/**
	 * Called by layoutLoadedComplete.
	 * Remove the layout from the layouts array and unset its onUnload function.
	 * @param silexLayout	the layout to unregister
	 */
	function unRegisterLayout(silexLayout:org.silex.core.Layout) {
		silex_ptr.utils.trace("org.silex.core.Application unRegisterLayout - silexLayout="+silexLayout);
		for (var idx:Number=0;idx<layouts.length;idx++){
			if (layouts[idx]==silexLayout)
				layouts.splice(idx,1);
		}
		this._parent.onUnload=undefined;
	}
	/**
	 * Find the layout object (silexLayout) at the root of the layout containing a given MovieClip.
	 * @param target_mc	the movie clip of which we want to find the layout object
	 * @return the root of the layout containing a target_mc
	 */
	function getLayout(target_mc:MovieClip):org.silex.core.Layout{
		//silex_ptr.utils.trace("org.silex.core.Application getLayout: target_mc="+target_mc._target);
		// loop on target_mc parents
		var ptr_mc:MovieClip=target_mc;
		while(ptr_mc){
			// is there a silexLayout object?
			if (ptr_mc.silexLayout){
				//silex_ptr.utils.trace("org.silex.core.Application getLayout: silexLayout object was found - target_mc="+target_mc._target+" - silexLayout="+ptr_mc.silexLayout+" , "+ptr_mc.silexLayout._target+" , "+ptr_mc.silexLayout.sectionName);
				return ptr_mc.silexLayout;
			}
			ptr_mc=ptr_mc._parent;
		}
		// here, no silexLayout object was found
		silex_ptr.utils.trace("org.silex.core.Application getLayout ERROR: no silexLayout object was found - target_mc="+target_mc._target);
	}
	/**
	 * Save a section. 
	 * Build the XML object and send it to the db.
	 * @param	target_mc	any clip contained in the section to be saved.
	 */
	function save (target_mc:MovieClip){
		// call hooks
        var hookManager:HookManager = HookManager.getInstance();
		hookManager.callHooks(ApplicationHooks.SAVE_START_HOOK_UID, arguments);
		
		// retrieve layout object containing target_mc
		var layout:org.silex.core.Layout=getLayout(target_mc);

		// build layout dom
		silex_ptr.dynDataManager.buildLayoutDom(layout);
		
		// convert to xml
		var xmlData:String=silex_ptr.xmlDom.objToXml(layout.dom);
		var sectionName:String=silex_ptr.utils.cleanID(layout.sectionName);
		var xmlFileName:String=layout.xmlFileName;
		
		// html tags
		var seoData_obj:Object=getSeoData(layout);
		//var htmlTags_xml:String=silex_ptr.xmlDom.objToXml(htmlTags_obj);
		
		// write xml
		silex_ptr.com.writeSectionData(xmlData, xmlFileName, sectionName, seoData_obj, Utils.createDelegate(this,saveDone),layout.dom);
		
		// set layout as clean
		layout.isDirty=false;
		hookManager.callHooks(ApplicationHooks.SAVE_END_HOOK_UID, null);
	}
	/**
	 * Callback function executed when a save operation is done.
	 * @param	error_str	the description of the error. null if no error occured.
	 */
	function saveDone (error_str:String) {
		if (error_str && error_str!=""){
			// handle error while saving
			this.silex_ptr.utils.trace("org.silex.core.Application saveDone - error while saving, data is not saved: "+error_str,silex_ptr.config.DEBUG_LEVEL_ERROR);
			this.silex_ptr.utils.alert(silex_ptr.utils.revealAccessors(silex_ptr.config.ERROR_PAGE_SAVE_ERROR,{error:error_str}));
			dispatchEvent("unsaved");
		}
		else{
			this.silex_ptr.utils.trace("org.silex.core.Application saveDone - section saved");
			dispatchEvent("saved");
		}
	}
	/**
	 * Build an indexable object for seo, search engine, rss.
	 * @return an object with the properties title (string), link (string), pubDate (Date) and content (array of objects returned by ui.players.UiBase::getSeoData)
	 */
	function getSeoData(layout:org.silex.core.Layout):Object {
		// **
		// layout hand written keywords
		
		// **
		// automatic keywords
		// layoutPath used for href
		var layoutPath:String=silex_ptr.deeplink.getLayoutPath(layout).join("/");;
		// relative path : if path = a/b/c then relative path = ../../../
		var nSlashes:Number=layoutPath.split("/").length-1;

		var layoutRelativePath:String="../../";
		for (var idx:Number=0;idx<nSlashes;idx++) layoutRelativePath+="../";
		
		// res_obj to store result
		var res_obj:Object=new Object;
		res_obj.title=layout.sectionName;
		res_obj.deeplink=layoutPath;
		res_obj.pubDate=(new Date()).toString();
		res_obj.urlBase=silex_ptr.utils.getRootUrl()+silex_ptr.config.id_site+"/";
		
		// browse all players
		res_obj.content=new Array;
		for (var layerIdx:Number=0;layerIdx<layout.layers.length;layerIdx++){
			for (var playerIdx:Number=0;playerIdx<layout.layers[layerIdx].players.length;playerIdx++){
				// retrieve the player reference
				var playerName_str:String=layout.layers[layerIdx].players[playerIdx]._name;
				var player/*:org.silex.ui.UiBase*/=layout.layers[layerIdx][playerName_str].main;
				// get the tags
				//silex_ptr.utils.trace("org.silex.core.Application getSeoData "+player+" - "+player.getSeoData);
				var playerSeoData:Object=player.getSeoData(layoutRelativePath);
				if (playerSeoData && (playerSeoData.text || playerSeoData.tags || playerSeoData.description || playerSeoData.links || playerSeoData.htmlEquivalent || playerSeoData.className))
					res_obj.content.push(playerSeoData);
			}
		}
		//silex_ptr.utils.trace("org.silex.core.Application getSeoData - "+res_obj.description+" - "+res_obj.htmlEquivalent);
		return res_obj;
	}
	
	/**
	 * Use org.silex.ui.UiBase::playerName to retrieve the player named playerName_str in the players tool box.
	 * @param playerName_str	the name of the player to find
	 * @return the player named playerName_str in the players tool box
	 */
	function getPlayerByName(playerName_str:String)/*:org.silex.ui.UiBase*/{
		// parse the org.silex.core.Layout objects
		var layout_ptr:org.silex.core.Layout=layoutContainer.silexLayout;
		while (layout_ptr){
			// parse the ui.Layer objects
			for (var layerIdx:Number=0; layerIdx<layout_ptr.layers.length;layerIdx++){
				// parse the players array ({_name:...,type:...})
				for (var playerIdx=0;playerIdx<layout_ptr.layers[layerIdx].players.length;playerIdx++){
					// check if it is the targeted player
					var player=layout_ptr.layers[layerIdx][layout_ptr.layers[layerIdx].players[playerIdx]._name].main;
					if (player.playerName==playerName_str){
						return player;
					}
				}
			}
			// next layout object
			layout_ptr=layout_ptr.currentChildLayoutContainer.silexLayout;
		}
		// no player was found
		return null;
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Group: fonts
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Store all fonts in silex_ptr.config.embeddedFont array.
	 */
	function registerEmbeddedFont(fontName_str:String){
		if (!silex_ptr.config.embeddedFont)
			silex_ptr.config.embeddedFont=[];
		else{
			// check if it is allready registered
			for (var idx:Number=0;idx<silex_ptr.config.embeddedFont.length;idx++){
				if (silex_ptr.config.embeddedFont[idx]==fontName_str){
					// this font is allready registerd
					return;
				}
			}
		}
		silex_ptr.config.embeddedFont.push(fontName_str);
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Group: screen saver
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * Initialize the screen saver system.
	 */
	function screenSaverInit(){
		//silex_ptr.utils.trace("org.silex.core.Application screenSaverInit "+silex_ptr.config.screenSaverDelay);
		if (screenSaverIntervalIdx>=0){
			// interval
			clearInterval(screenSaverIntervalIdx);
			screenSaverIntervalIdx=-1;
			// screen saver mode
			screenSaverMode=SCREEN_SAVER_MODE_OFF;
			
			// mouse
			Mouse.removeListener(screenSaverMouseListener);
			delete screenSaverMouseListener;
		}
		if (parseFloat(silex_ptr.config.screenSaverDelay)>0){
			// interval
			screenSaverIntervalIdx=setInterval(Utils.createDelegate(this,screenSaverActivate),parseFloat(silex_ptr.config.screenSaverDelay)*1000);
			// screen saver mode
			screenSaverMode=SCREEN_SAVER_MODE_ON;
			// mouse
			screenSaverMouseListener=new Object;
			screenSaverMouseListener.onMouseDown=screenSaverMouseListener.onMouseMove=screenSaverMouseListener.onMouseUp=screenSaverMouseListener.onMouseWheel=Utils.createDelegate(this,screenSaverSignal);
			Mouse.addListener(screenSaverMouseListener);
		}
	}	
	/**
	 * Turn back to normal mode because activity has been detected.
	 * Called by screenSaverSignal.
	 * @see	org.silex.core.Application::screenSaverSignal
	 */
	function screenSaverDeactivate(){
		//silex_ptr.utils.trace("org.silex.core.Application screenSaverDeactivate "+silex_ptr.config.screenSaverDeactivateCommands);

		// initialize again
		screenSaverInit();
		
		if (silex_ptr.authentication.isLoggedIn()==false){
			// execute commands (separated with "|")
			var commands_array:Array=silex_ptr.config.screenSaverDeactivateCommands.split("|");
			for (var idx:Number=0;idx<commands_array.length;idx++)
				silex_ptr.interpreter.exec(silex_ptr.utils.revealAccessors(commands_array[idx],silex_ptr),layouts[0]);
		}
	}
	/**
	 * Turn into screen saver mode because there was no activity for a certain amount of time.
	 * Only if user is not logged in.
	 * Called by the system (setInterval callback).
	 */
	function screenSaverActivate(){
		//silex_ptr.utils.trace("org.silex.core.Application screenSaverActivate "+silex_ptr.config.screenSaverActivateCommands);

		// screen saver mode
		screenSaverMode=SCREEN_SAVER_MODE_ACTIVE;
		
		// interval
		clearInterval(screenSaverIntervalIdx);
		screenSaverIntervalIdx=-1;
		
		if (silex_ptr.authentication.isLoggedIn()==false){
			// execute commands (separated with "|")
			var commands_array:Array=silex_ptr.config.screenSaverActivateCommands.split("|");
			for (var idx:Number=0;idx<commands_array.length;idx++)
				silex_ptr.interpreter.exec(silex_ptr.utils.revealAccessors(commands_array[idx],silex_ptr),layouts[0]);
		}
		else{
			silex_ptr.utils.alertSimple(silex_ptr.config.MESSAGE_SCREEN_SAVER_LOGGED_IN);
		}
	}
	/**
	 * Delay the screen saver activation because activity is detected.
	 * Called by SILEX when activity is detected.
	 */
	function screenSaverSignal(){
		//silex_ptr.utils.trace("org.silex.core.Application screenSaverSignal "+screenSaverMode);
		if (screenSaverMode==SCREEN_SAVER_MODE_ON){
			screenSaverInit();
		}
		else if (screenSaverMode==SCREEN_SAVER_MODE_ACTIVE){
			screenSaverDeactivate();
		}
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Group: layers
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	/**
	 * Called by org.silex.core.Api.config object (watch on globalContext).
	 */
	function refreshAllLayers(){
		silex_ptr.utils.trace("org.silex.core.Application refreshAllLayers ");
		for (var idxLayout:Number=0;idxLayout<layouts.length;idxLayout++){
			for (var idxLayer:Number=0;idxLayer<layouts[idxLayout].layers.length;idxLayer++)
				layouts[idxLayout].layers[idxLayer].refresh();
		}
	}
	/**
	 * Check if one of the target's child layout is dirty, 
	 * @param	targetLayout	the layout to be checked
	 * @return true if targetLayout has changed. null otherwise.
	 */
	function isAChildDirty(targetLayout:org.silex.core.Layout):Boolean{
		// working index
		var layoutIdx:Number=0;
		// go to the target in the layouts array
		while (layouts[layoutIdx]){
			if (layouts[layoutIdx]==targetLayout)
				break;
			layoutIdx++;
		}
		// for all children, check if it is dirty
		layoutIdx++;
		while (layouts[layoutIdx]){
			if (layouts[layoutIdx].isDirty==true)
				return layouts[layoutIdx];
			layoutIdx++;
		}
		return null;
	}
}