﻿/*
	this file is part of SILEX
	SILEX : RIA developement tool - see http://silex.sourceforge.net/

	SILEX is (c) 2004-2007 Alexandre Hoyau and is released under the GPL License:

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License (GPL)
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/
/* 
*************************************************
    SILEX API 2004-2005
**************************************************
   Title:       Filter.as
   Description: filter SILEX accessors

   Copyright:   Copyright (c) 2004
   Author:      Alex. H. lex@silex.tv
   Version:     1
*/
/**
 * This class is used to filter SILEX accessors. It is a singleton pattern.
 */
//import org.silex.core.Utils;
class org.silex.core.Filter
{
	/**
	 * Reference to silex main Api object (org.silex.core.Api).
	 */
	private var silex_ptr:org.silex.core.Api;
	/**
	 * Constructor.
	 * @param	api	reference to silex main Api object (org.silex.core.Api)
	 */
	function Filter(api:org.silex.core.Api) {
		// api reference
		silex_ptr=api;
		
		silex_ptr.utils.trace("org.silex.core.Filter Constructor");
	}
	/**
	 * test if a filter exists
	 * @param	filter_name_str
	 * @return
	 */
	function filterExist(filter_name_str:String):Boolean 
	{
		if (this[filter_name_str])
			return true;
		else
			return false;
	}
	/**
	 * Call the method corresponding to the desired filter.
	 * @param filter_name_str	name of the filter
	 * @param value_to_filter	value which has to be filtered
	 * @param parameters_array	optionnal parameters
	 * @param initial_source_mc	optionnal source object
	 * @return	null if an error occured or the filter value
	 */
    function exec(filter_name_str:String,value_to_filter,parameters_array:Array,initial_source_mc:Object)
    {
		// non optionnal value
		if (!value_to_filter){
			// error: no value specified
            silex_ptr.utils.trace("org.silex.core.Filter exec - error: no value specified");
            return null;
		}
		
		// non optionnal value
        if (!filter_name_str || filter_name_str == "") {
			// error: no filter specified
            silex_ptr.utils.trace("org.silex.core.Filter exec - error: no filter specified");
            return null;
		}
		
		// parameters_array is optionnal
        if (!parameters_array)
			parameters_array = new Array;
			
		// initial_source_mc is optionnal
        if (!initial_source_mc)
			initial_source_mc = this;

		silex_ptr.utils.trace("org.silex.core.Filter exec - "+filter_name_str+" in "+initial_source_mc);
		
		// add the value_to_filter as the 1st argument
		parameters_array.unshift(value_to_filter);
		
		// call the method
		if (this[filter_name_str]){
			return this[filter_name_str].apply(initial_source_mc,parameters_array)
		}
		// error: filter does not exist
		silex_ptr.utils.trace("org.silex.core.Filter exec - error: filter does not exist ("+filter_name_str+")");
		return null;
    }

    /////////////////////////////////////////////////
    // FILTERS
	// in these methods, "this" is the source object
	// the 1st argument is the value to filter
    /////////////////////////////////////////////////
	/**
	 * URLencode the string
	 * @example		<<urlencode silex.rootUrl>> returns the urlencoded version of the url of the server
	 */
    function urlencode(value_to_filter:String):String {
		return escape(value_to_filter);
    }
	/**
	 * double URLencode the string
	 * @example		<<urlencode silex.rootUrl>> returns the 2x urlencoded version of the url of the server
	 */
    function urlencode2(value_to_filter:String):String {
		return escape(escape(value_to_filter));
    }
}