﻿/*
	this file is part of SILEX
	SILEX : RIA developement tool - see http://silex-ria.org/

	SILEX is (c) 2004-2007 Alexandre Hoyau and is released under the GPL License:

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License (GPL)
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/
import org.silex.core.Utils;
import mx.controls.List;

/**
 * Players toolbox
 */
class org.silex.toolbox.Players extends org.silex.toolbox.Toolbox {

	static var REFRESH_INTERVAL_MS:Number = 500;

	// event to change selection on click
	var playerListener_obj:Object;

	// refresh interval
	var intervalIdx:Number=-1;

	
	/**
	 * Constructor.
	 * Add a listener on change event of toolbox layers and call this onLoad method.
	 * Init UI buttons.
	 */
	public function Players(){

		// event to change selection on click
		playerListener_obj=new Object;
		playerListener_obj.onRelease=Utils.createDelegate(this,togglePlayerSelection);

		//trace("New Players");
		ui_mc.select_btn.onRelease = Utils.createDelegate(this,select);
		ui_mc.deselect_btn.onRelease = Utils.createDelegate(this,deselect);
		ui_mc.invert_btn.onRelease = Utils.createDelegate(this,invert);
		ui_mc.show_btn.onRelease = Utils.createDelegate(this,show);
		ui_mc.hide_btn.onRelease = Utils.createDelegate(this,hide);
		ui_mc.solo_btn.onRelease = Utils.createDelegate(this,solo);
		ui_mc.up_btn.onRelease = Utils.createDelegate(this,up);
		ui_mc.down_btn.onRelease = Utils.createDelegate(this,down);
		ui_mc.toTop_btn.onRelease = Utils.createDelegate(this,toTop);
		ui_mc.toBottom_btn.onRelease = Utils.createDelegate(this,toBottom);
		ui_mc.del_btn.onRelease = Utils.createDelegate(this,deleteSelection);
		ui_mc.help_btn.onRelease = function(){_root.getURL(_global.getSilex().config.HELP_URL_DOCUMENTATION_ROOT+_global.getSilex().config.HELP_URL_WYSIWYG_PLAYERS,"_blank");};
				
		// listener for application background
		var applicationListener:Object=new Object;
		applicationListener.onBgPress=Utils.createDelegate(this,deselect);
		silex_ptr.application.addEventListener("onBgPress",applicationListener);
		applicationListener.playersLoaded=Utils.createDelegate(this,refresh);
		silex_ptr.application.addEventListener("playersLoaded",applicationListener);
		
		//  refresh regularly to display all layers anytime
		intervalIdx=setInterval(Utils.createDelegate(this, refresh),REFRESH_INTERVAL_MS,false);
	}
	function onUnload(){
		super.onUnload();
		clearInterval(intervalIdx);
	}
	function allToolsLoaded () {
		super.allToolsLoaded();
			var refreshListener = new Object();
			refreshListener.change = Utils.createDelegate(this,refresh);
			_parent._parent.layers_tool.main.addEventListener("change",refreshListener);
	}

	// to be documented : togglePlayerSelection
	// event to change selection on click
	function togglePlayerSelection(evt:Object)
	{
		//silex_ptr.utils.trace("toolbox.Players togglePlayerSelection "+evt.target._name+" - "+list.dataProvider.length+" - "+list.length+" -- "+list.dataProvider);
//		for(var idx:Number=0;idx<list.dataProvider.length;idx++)
		for(var idx:Number=0;idx<list.length;idx++)
//		for(var idx in list.dataProvider)
		{
			var listItem:Object=list.getItemAt(idx);
			//var listItem:Object=list.dataProvider[idx];
			//silex_ptr.utils.trace("toolbox.Players togglePlayerSelection element at "+idx+" = "+listItem.label);
			// bug: if (listItem.data==evt.target)
			if (listItem._name==evt.target._parent._name)
			{
				//silex_ptr.utils.trace("toolbox.Players togglePlayerSelection found target at "+idx);
				list.selectedIndex=idx;
			}
		}
		dispatchEvent({type:"change"});
	}
	
	// to be documented : deleteSelection
	function deleteSelection()
	{
		if (!list.selectedItem){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		silex_ptr.utils.confirm("Do you really want to delete the selection?",Utils.createDelegate(this,deleteSelectionCbkFunction));
	}
	function deleteSelectionCbkFunction(confirm_bool:Boolean) {
		if (confirm_bool==true){
			_parent._parent.layers_tool.main.getSelectedContent()[list.selectedItem._name].main.unloadMovie();
			_parent._parent.layers_tool.main.getSelectedContent().players.splice(list.selectedIndex,1);
			// bug : _parent._parent.layers_tool.main.getSelectedContent().refresh();
			// to redesign : 
			_parent._parent.layers_tool.main.dispatchEvent({type:"change"});
		}
	}
	
	public function onLoad(){
		super.onLoad();
		
		title=silex_ptr.config.TOOL_BOX_TITLE_PLAYERS;
		//list.labelField="playerName";
		refresh();
		// Defined the change event for the list.
		var listListener:Object = new Object();
		listListener.change = Utils.createDelegate(this, change);
		list.addEventListener("change", listListener);
		// icons and item function
		list.iconFunction=Utils.createDelegate(this,iconFunction);
	}
	function iconFunction(item:Object) {
		var layer=_parent._parent.layers_tool.main.getSelectedContent();
		var player/*:org.silex.ui.players.PlayerBase*/=layer[item._name].main;
		if (player.isVisible==true) return "hideButton";
		return "showButton";
	}
	
	/**
	 * Check the player list by the getSelectedContent method of layers toolbox. 
	 * Defined the change event for the list.
	 */

	public function refresh(doDispatch:Boolean){
		var selectedIndex:Number=list.selectedIndex;

		var layer=_parent._parent.layers_tool.main.getSelectedContent();
		list.removeAll();
		list.dataProvider= new Array;
		for(var i=0; i<layer.players.length; i++){
			layer.players[i].label=layer[layer.players[i]._name].main.playerName;
			//silex_ptr.utils.trace("toolbox.Players refresh "+layer.players+"["+i+"] = "+layer.players[i]._name);
			list.addItem(layer.players[i]);
			
			// event to change selection on click
			layer[layer.players[i]._name].main.addEventListener("onRelease",playerListener_obj);
		}
		if (doDispatch!=false){
			super.refresh();
			dispatchEvent({type:"change"});
		}
		else{
			// keep selected item
			list.selectedIndex=selectedIndex;
		}
	}

	/**
	 * Function used to retrieve the selected layers.
	 */
	public function getSelectedContent():Array {
		return list.selectedItem;
	}
	
	/**
	 * Event function called by change tree component. Dispatch event in toolbox players.
	 */
	public function change(evt_obj:Object):Void {
		trace("change in Players "+list.selectedItem.label);
		dispatchEvent({type:"change"});
	}
	
	/**
	 * Function select. Not specified in wiki
	 */
	public function select():Void {
		trace("select");
		silex_ptr.utils.alertSimple("Not implemented yet");
		dispatchEvent({type:"change"});
	}
	
	/**
	 * empty the selection. Deselect the items selected.
	 */
	public function deselect():Void {
		trace("deselect");
		list.selectedIndex = null;
		dispatchEvent({type:"change"});
	}
	
	/**
	 * Invert selection. Deselect item selected and select item deselected
	 */
	public function invert():Void {
/*		trace("invert");
		for(var i=0 ; i<list.length ; i++){
			if (list.selectedIndices.indexOf(i)>-1)
				list.selectedIndices.splice(list.selectedIndices.indexOf(i),1);
			else
				list.selectedIndices.push(i);
		}
		dispatchEvent({type:"change"});
*/		silex_ptr.utils.alertSimple("Not implemented yet");
	}
	
	/**
	 * show selected players (use player::setVisible method)
	 * Check list.selectedItems and call the setVisible method with true for each item
	 */
	public function show(idx:Number):Void {
		if (idx==undefined) idx=list.selectedItem;
		if (idx==undefined){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		var layer=_parent._parent.layers_tool.main.getSelectedContent();
		var player/*:org.silex.ui.players.PlayerBase*/=layer[list.getItemAt(idx)._name].main;
		player.setVisible(true);
		player.temporarilyVisibleInAdminMode = true;
		dispatchEvent({type:"change"});
	}
	
	/**
	 * hide selected players (use player::setVisible method) 
	 * Check list.selectedItems and call the setVisible method with false for each item
	 */
	public function hide(idx:Number):Void {
		//silex_ptr.utils.trace("toolbox.Players hide "+idx+" or "+list.selectedItem);
		if (idx==undefined) idx=list.selectedItem;
		if (idx==undefined){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		var layer=_parent._parent.layers_tool.main.getSelectedContent();
		var player/*:org.silex.ui.players.PlayerBase*/=layer[list.getItemAt(idx)._name].main;
		player.setVisible(false);
		player.temporarilyVisibleInAdminMode = false;
		dispatchEvent({type:"change"});
	}
	
	/**
	 * show only selected players (use player::setVisible method) 
	 * Check list.selectedItems and call the setVisible method with true for each item and with false for the others
	 */
	public function solo():Void {
		if (!list.selectedItem){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		if (list.selectedItem.isSolo==undefined) list.selectedItem.isSolo=false;
		list.selectedItem.isSolo=!list.selectedItem.isSolo;
		
		var indexSelected = list.selectedIndex;
		for(var i=0 ; i<list.length ; i++){
			if (i == indexSelected) {
				list.getItemAt(i).setVisible(true);
				list.getItemAt(i).temporarilyVisibleInAdminMode = true;
			}
			else {
				list.getItemAt(i).setVisible(!list.selectedItem.isSolo);
				list.getItemAt(i).temporarilyVisibleInAdminMode = !list.selectedItem.isSolo;
			}
		}
		dispatchEvent({type:"change"});
	}
	
	/**
	 * edit the selected layer (one layer at a time) (set all players setEditable method with true)
	 */
	public function edit():Void {
		trace("edit");
		silex_ptr.utils.alertSimple("Not implemented yet");
		dispatchEvent({type:"change"});
	}
	
	/**
	 * push up z order decrease the selectedItemIndex
	 */
	public function up():Void {
		trace("up");
		if (!list.selectedItem){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		if(list.selectedIndex>0){
			var indexSelected = list.selectedIndex;
			list.addItemAt(indexSelected-1,list.getItemAt(indexSelected));
			list.removeItemAt(indexSelected+1);
			list.selectedIndex = indexSelected-1;
			
			var layer = _parent._parent.layers_tool.main.getSelectedContent().players;
			layer.splice(indexSelected-1,0,layer[indexSelected]);
			layer.splice(indexSelected+1,1);
			
			dispatchEvent({type:"change"});
		}
		_parent._parent.layers_tool.main.getSelectedContent().refresh();
	}
	
	/**
	 * push down z order increase the selectedItemIndex
	 */
	public function down():Void {
		trace("down");
		if (!list.selectedItem){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		if(list.selectedIndex<list.length-1){
			var indexSelected = list.selectedIndex;
			list.addItemAt(indexSelected+2,list.getItemAt(indexSelected));
			list.removeItemAt(indexSelected);
			list.selectedIndex = indexSelected+1;
			
			var layer = _parent._parent.layers_tool.main.getSelectedContent().players;
			layer.splice(indexSelected+2,0,layer[indexSelected]);
			layer.splice(indexSelected,1);
			dispatchEvent({type:"change"});
		}
		_parent._parent.layers_tool.main.getSelectedContent().refresh();
	}
	
	/**
	 * push to top set to 0 the selectedItemIndex
	 */
	public function toTop():Void {
		trace("toTop");
		if (!list.selectedItem){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		if(list.selectedIndex>0){
			var indexSelected = list.selectedIndex;
			list.addItemAt(0,list.getItemAt(indexSelected));
			list.removeItemAt(indexSelected+1);
			list.selectedIndex = 0;
			
			var layer = _parent._parent.layers_tool.main.getSelectedContent().players;
			layer.splice(0,0,layer[indexSelected]);
			layer.splice(indexSelected+1,1);
			
			dispatchEvent({type:"change"});
		}
		_parent._parent.layers_tool.main.getSelectedContent().refresh();
	}
	
	/**
	 * push to bottom set to list.lenght the selectedItemIndex
	 */
	public function toBottom():Void {
		trace("toBottom");
		if (!list.selectedItem){
			silex_ptr.utils.alertSimple(silex_ptr.config.ERROR_NO_SELECTION);
			return;
		}
		if(list.selectedIndex<list.length-1){
			var indexSelected = list.selectedIndex;
			list.addItemAt(list.length,list.getItemAt(indexSelected));
			list.removeItemAt(indexSelected);
			list.selectedIndex = list.length-1;
			
			var layer = _parent._parent.layers_tool.main.getSelectedContent().players;
			layer.splice(list.length,0,layer[indexSelected]);
			layer.splice(indexSelected,1);

			dispatchEvent({type:"change"});
		}
		_parent._parent.layers_tool.main.getSelectedContent().refresh();
	}
}