﻿/*
	this file is part of SILEX
	SILEX : RIA developement tool - see http://silex-ria.org/

	SILEX is (c) 2004-2007 Alexandre Hoyau and is released under the GPL License:

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License (GPL)
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/
// for sortItem etc, look at mx.controls.listclasses.DataSelector

import mx.transitions.Tween;
import mx.transitions.easing.*;

import mx.events.EventDispatcher;
import mx.controls.listclasses.DataProvider;
import mx.managers.DepthManager
import org.silex.core.Utils;

class org.silex.toolbox.list.CustomList extends mx.core.UIObject{

	///////////////////////////////////
	// Attributes
	///////////////////////////////////
	// attributes from the mx.controls.listclasses.DataSelector class
	var lastSelID : Number;
	var lastSelected;
	
	// UI
	var listContent:MovieClip;
	var mask_mc:MovieClip;
	var bg_mc:MovieClip;
	
	// 
	
	/* __dataProvider
	 * data provider object
	 */
	private var __dataProvider:Array;
	
	/* dataProvider
	 * setter and getter for the data provider object
	 */
	public function get dataProvider ():Array {
		return __dataProvider;
	}
	public function set dataProvider (_dp:Array) {
		if(__dataProvider!=_dp){
			trace("CustomList set dataProvider "+_dp);
			// stop listening the "modelChanged" event of the OLD data provider object
			if (__dataProvider){
				__dataProvider["removeEventListener"]("modelChanged",redraw);
			}
			// store the new data provider
			__dataProvider=_dp;
			// listen the "modelChanged" event of the DataProvider class
			__dataProvider.addEventListener("modelChanged",Utils.createDelegate(this,redraw));
			redraw();
			if (alignToTop==true)
				listContent.y=listContent._height;
		}
	}
	/** selectedIndex
	 * the selected index of a single-selection list.
	 */
	var _selectedIndex:Number=-1;
	function get selectedIndex():Number {
		return _selectedIndex;
	}
	function set selectedIndex (val:Number) {
		// store the new value
		_selectedIndex=val;
		// update selected item
		_selectedItem=getItemAt(val);
		// dispatch event
		dispatchEvent({type:"change",target:this});
		// refresh display
		refreshCells();
	}

	/** selectedItem
	 * the selected index of a single-selection list.
	 */
	var _selectedItem:Object;
	function get selectedItem():Object {
		return _selectedItem;
	}
	function set selectedItem (val:MovieClip) {
		// store the new value
		_selectedItem=val;
		// update selected index
		_selectedIndex=getItemIndex(val);
		// dispatch event
		dispatchEvent({type:"change",target:this});
		// refresh display
		refreshCells();
	}


	
	/** nCells
	 * number of cells, depends on the size of the list
	 * the number of rows that are at least partially visible in the list. 
	 */
	var rowCount:Number=0;
	
	/** rowSize
	 * the height, in pixels, of every row in the list
	 */
/*		For normal List:
	[Inspectable(defaultValue=20)]
	var rowSize:Number;
*/

	/** isHorizontal
	 * true if the list is horizontal
	 * false if the list is vertical
	 */
	[Inspectable(defaultValue=true)]
	var isHorizontal:Boolean;
	
	/** cellRenderer
	 * assigns the cell renderer to use for each row of the list. This property must be a symbol linkage identifier. Any class used for this property must implement the CellRenderer API.
	 */
	[Inspectable(defaultValue="defaultCellRenderer")]
	var cellRenderer:String;

	/** scrollBarLinkageName
	 * This property must be a symbol linkage identifier.
	 */
	[Inspectable(defaultValue="", name="scroll bar linkage name")]
	var scrollBarLinkageName:String;
	var scrollBar_mc:MovieClip;
	
	/** labelField
	 * specifies a field in each item to be used as display text.
	 */
	[Inspectable(defaultValue="label")]
	var labelField:String;

	// alignToTop => the list will show its top
	[Inspectable(defaultValue=false)]
	var alignToTop:Boolean;

	/** __width and __height
	 * 
	 */
	var __width:Number;
	var __height:Number;
	
	///////////////////////////////////
	// Methods
	///////////////////////////////////
	
	// constructor
	function CustomList () {
		// add ability to broadcast events.
		EventDispatcher.initialize(this);
		
		DataProvider.Initialize(Array);
		dataProvider=new Array;
	}

	///////////////////////////////////
	// List class methods
	///////////////////////////////////
	// Adds an item to the end of the list.
	function addItem(_obj){
		dataProvider.addItem(_obj);
		redraw();
	}
	
	// Adds an item to the list at the specified index.
	function addItemAt(idx:Number){
		redraw();
	}
	
	// Returns the item at the specified index.
	function getItemAt(idx:Number){
		listContent["listRow"+idx.toString()];
	}
	
	// Removes all items from the list.
	function removeAll(){
		redraw();
	}
	
	// Removes the item at the specified index.
	function removeItemAt(){
		redraw();
	}
	
	// Replaces the item at the specified index with another item.
	function replaceItemAt(){
		redraw();
	}
	
	// Applies the specified properties to the specified item.
	function setPropertiesAt(){
		redraw();
	}
	
	// Sorts the items in the list according to the specified compare function.
	function sortItems(func){
		lastSelID = __dataProvider.getItemID(lastSelected);
		__dataProvider.sortItems(func);
		redraw();
	}
	
	// Sorts the items in the list according to a specified property.
	function sortItemsBy(fieldName, order) : Void
	{
		lastSelID = __dataProvider.getItemID(lastSelected);
		__dataProvider.sortItemsBy(fieldName, order);
		redraw();
	}
	///////////////////////////////////
	// UIObject class methods
	///////////////////////////////////
	var onLoadOccured:Boolean=false;
	function onLoad () {
		onLoadOccured=true;
		
		trace("CustomList onLoad");
		//listContent=createEmptyMovieClip("listContent",getNextHighestDepth());//DepthManager.kTop);//
		listContent=attachMovie("ListScrollableContent","listContent",getNextHighestDepth());
		listContent.setMask(MovieClip(mask_mc));
		listContent.y=0;
		listContent.offset=mask_mc._height;
		
		var w:Number=_width;
		var h:Number=_height;
		
		if (scrollBarLinkageName!=""){
			// bottom up list
			// scrollBar_mc=attachMovie(scrollBarLinkageName,"scrollBar_mc",getNextHighestDepth(),{_width:_width,_height:_height,listBgName:"mask_mc",containerName:"listContent",isVertical:true,_rotation:180,_y:mask_mc._height});
			scrollBar_mc=attachMovie(scrollBarLinkageName,"scrollBar_mc",getNextHighestDepth(),{_width:_width,_height:_height,listBgName:"mask_mc",containerName:"listContent",isVertical:true});
			scrollBar_mc.propertyToScroll="y";
		}
		
		setSize(w,h);
		
		if (alignToTop==true)
			listContent.y=listContent._height;
			
			

		Mouse.addListener(this);
	}
	// mouse wheel
	function onMouseWheel(delta) {
	//    list.listContent._y += delta*10;
		vPosition-= delta*1;
	//	setScroll(list.vPosition+delta);
	//	trace(list.vPosition+delta);
	}
	function get vPosition():Number {
		// compute average height of the cells
		var averageHeight:Number=listContent._height/rowCount;
		
		// return the first visible item
		return -listContent.y/averageHeight;
	}
	function set vPosition(_num:Number) {
		trace("CustomList set vPosition "+_num);
		// compute average height of the cells
		var averageHeight:Number=listContent._height/rowCount;
		trace("CustomList set vPosition averageHeight="+averageHeight);
		
		listContent.y=-Math.round(_num*averageHeight);

		if (scrollBarLinkageName!=""){
			scrollBar_mc.redraw();
		}
	}

	// redraw the component
	function redraw () {
		trace("CustomList redraw");
		
		// For custom List
		adjustCellsCount();
		refreshCells();

		listContent.offset=mask_mc._height;
		
		if (scrollBarLinkageName!=""){
			scrollBar_mc.container_mc=listContent;
			scrollBar_mc.listBg_mc=mask_mc;
			scrollBar_mc.redraw();
		}
	}
	// resizes the object to the requested size.
	function setSize (w:Number,h:Number) {
		trace("CustomList setSize "+w+" x "+h);
		
		_xscale=_yscale=100;
		
		__width=w;
		__height=h;
		
		// set mask size
		bg_mc._width=w;
		bg_mc._height=h;
		mask_mc._width=w;
		mask_mc._height=h;
		
		// For normal List:
		//adjustCellsCount();
		
		if (scrollBarLinkageName!=""){
			scrollBar_mc.container_mc=listContent;
			scrollBar_mc.listBg_mc=mask_mc;
			scrollBar_mc._x=mask_mc._x+mask_mc._width+10;
			// bottom up list
			//scrollBar_mc._rotation=180;
			//scrollBar_mc._y=mask_mc._y+mask_mc._height;
			scrollBar_mc._y=mask_mc._y;
			scrollBar_mc.setSize(w,h);
		}
	}
	
	// moves the object to the requested position. 
	function move (x:Number,y:Number) {
		trace("CustomList move");
	}
	
	/////////////////////////////////////
	// Private methods
	////////////////////////////////
	function adjustCellsCount () {
		
		if (!onLoadOccured) return;
		
		// **
		// update the number of rows

		// store old row count
		var oldRowCount:Number=rowCount;
		
		// compute the number of rows in the view
/*		For normal List:
		if (isHorizontal==true)
			rowCount=Math.ceil(__width/rowSize);
		else
			rowCount=Math.ceil(__height/rowSize);
		*/
		// For custom List
		rowCount=__dataProvider.length;
		
		// remove rows
		while (oldRowCount>rowCount){
			trace("CustomList adjustCellsCount removeMovieClip listRow"+(oldRowCount-1));
			listContent["listRow"+(oldRowCount-1)].removeMovieClip();
			oldRowCount--;
		}
		
		// default cell renderer
		if (cellRenderer=="") cellRenderer="defaultCellRenderer";
		
		// add rows
		while (oldRowCount<rowCount){
			trace("CustomList adjustCellsCount attachMovie - "+cellRenderer+" ,listRow"+oldRowCount);

			// create a new cell
			var cell_mc:MovieClip=listContent.attachMovie(cellRenderer,"listRow"+oldRowCount,listContent.getNextHighestDepth());//listContent.DepthManager.kTop);//
			
			// initialize the new cell
			cell_mc.listOwner=this;
			
			// For normal List:
			//cell_mc.setValue(__dataProvider[oldRowCount][labelField], __dataProvider[oldRowCount], (selectedIndex==oldRowCount));
			
			// update oldRowCount
			oldRowCount++;
		}
	}
	function refreshCells () {
		// For custom List:
/**/		for (var idx:Number=0;idx<__dataProvider.length;idx++){
			// cell data
			listContent["listRow"+idx].setValue(__dataProvider[idx][labelField], __dataProvider[idx], (selectedIndex==idx));
			// cell position
			var cell_mc:MovieClip=listContent["listRow"+idx];
			var previousCell_mc:MovieClip=listContent["listRow"+(idx-1)];
			if (isHorizontal){
				// beside the previous cell
				if (previousCell_mc)
					cell_mc._x=previousCell_mc._x+previousCell_mc._width;
				// centered vertically
				cell_mc.setSize(cell_mc._width,__height);
			}
			else{
				// beside the previous cell
				if (previousCell_mc)
					cell_mc._y=previousCell_mc._y+previousCell_mc._height;
				// centered horizontally
				cell_mc.setSize(__width,cell_mc._height);
			}
		}
		/**/
		///////////////////////////////
		// from bottom up
/*		for (var idx:Number=0;idx<__dataProvider.length;idx++){
			// cell data
			listContent["listRow"+idx].setValue(__dataProvider[idx][labelField], __dataProvider[idx], (selectedIndex==idx));
			// cell position
			var cell_mc:MovieClip=listContent["listRow"+idx];
			var previousCell_mc:MovieClip=listContent["listRow"+(idx-1)];
			
			// beside the previous cell
			if (previousCell_mc)
				cell_mc._y=previousCell_mc._y-cell_mc._height;
			// centered horizontally
			cell_mc.setSize(__width,cell_mc._height);
		}*/
	}
	function getItemIndex (cell_mc:MovieClip):Number {
		for (var idx:Number=0;idx<__dataProvider.length;idx++){
			if (listContent["listRow"+idx]==cell_mc)
				return idx;
		}
		return -1;
	}
}