﻿/*
	this file is part of SILEX
	SILEX : RIA developement tool - see http://silex-ria.org/

	SILEX is (c) 2004-2007 Alexandre Hoyau and is released under the GPL License:

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License (GPL)
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/
import org.silex.core.Utils;
import mx.transitions.Tween;
import mx.transitions.easing.*;


class org.silex.toolbox.list.CustomScrollBar extends mx.core.UIComponent {
	
	//////////////////////////////
	// params
	//////////////////////////////
	[Inspectable(defaultValue="up_btn",name="Up button name")]
	var upBtnName:String;

	[Inspectable(defaultValue="down_btn",name="Down button name")]
	var downBtnName:String;

	[Inspectable(defaultValue="bg_mc",name="List background name")]
	var listBgName:String;

	[Inspectable(defaultValue="container_mc",name="Container name")]
	var containerName:String;
	
	[Inspectable(defaultValue=500,name="Size of scroll")]
	var sizeOfScroll:Number;

	[Inspectable(defaultValue=true,name="Vertical scroll")]
	var isVertical:Boolean;

	// UI
	var bg_mc:MovieClip;
	var knob_mc:MovieClip;
	var up_btn:Button;
	var down_btn:Button;
	var listBg_mc:MovieClip;
	var container_mc:MovieClip;
	
	///
	var _tween:Tween;
	var propertyToScroll:String=""; // _x or _y
	
	function get scrollPosition():Number {
		trace("CustomScroll get scrollPosition "+container_mc+" - "+propertyToScroll+" - "+container_mc[propertyToScroll]);
		return container_mc[propertyToScroll];
	}
	function set scrollPosition(num:Number) {
		container_mc[propertyToScroll]=num;
	}
	function get maxScroll ():Number {
		var _num:Number;
		if (isVertical)
			_num=container_mc._height-listBg_mc._height;
		else
			_num=container_mc._width-listBg_mc._width;
		return _num;
	}


	
	//////////////////////////////////
	function onLoad () {
		if (propertyToScroll==""){
			if (isVertical==true)
				propertyToScroll="_y";
			else
				propertyToScroll="_x";
		}
			
		// UI
		up_btn=_parent[upBtnName];
		down_btn=_parent[downBtnName];
		listBg_mc=_parent[listBgName];
		container_mc=_parent[containerName];

		// events
		down_btn.onRelease=Utils.createDelegate(this,down);
		up_btn.onRelease=Utils.createDelegate(this,up);
		knob_mc.bg_mc.onPress=Utils.createDelegate(this,knobStartDrag);
		knob_mc.bg_mc.onRelease=knob_mc.knob_btn.onReleaseOutside=Utils.createDelegate(this,knobStopDrag);
		knob_mc.bg_mc.onRollOver=Utils.createDelegate(this,knobRollOver);
		knob_mc.bg_mc.onRollOut=Utils.createDelegate(this,knobRollOut);

		// redraw
//		setSize(_width,_height);
		redraw();
	}
	function knobRollOver () {
		setKnobSize(250,1.5);
	}
	function knobRollOut () {
		setKnobSize(100,5);
	}
	var _tweenX:Tween;
	var _tweenY:Tween;
	function setKnobSize (scale_num:Number,time_num:Number) {
			_tweenX=new Tween(knob_mc.knob_btn, "_xscale", Elastic.easeOut/*Strong.easeOut*/, knob_mc.knob_btn._xscale, scale_num, time_num, true);
			_tweenY=new Tween(knob_mc.knob_btn, "_yscale", Elastic.easeOut/*Strong.easeOut*/, knob_mc.knob_btn._yscale, scale_num, time_num, true);
	}

	function knobStartDrag() {
		knob_mc.startDrag(true,0,0,0,bg_mc._height);
		_tween.stop();
		onMouseMove=Utils.createDelegate(this,mousemove);
	}
	function mousemove() {
		//scrollPosition=-Math.round(maxScroll*knob_mc._y/bg_mc._height);
		setScroll(-Math.round(maxScroll*knob_mc._y/bg_mc._height),false);
	}

	function knobStopDrag(){
		onMouseMove=undefined;
		knob_mc.stopDrag();
		setScroll(-Math.round(maxScroll*knob_mc._y/bg_mc._height));
	}
	function redraw() {
		trace("CustomScroll redraw ");
		if (scrollPosition>0) scrollPosition=0;
		if (scrollPosition<-maxScroll) scrollPosition=-maxScroll;
		
		// is there enought content to scroll?
		if(maxScroll<0){
			knob_mc._y=scrollPosition=0;
			down_btn._visible=up_btn._visible=knob_mc._visible=_visible=false;
		}
		else{
			knob_mc._y=-bg_mc._height*scrollPosition/maxScroll;
			down_btn._visible=up_btn._visible=knob_mc._visible=_visible=true;
		}
	}
	
	function down() {
		setScroll (scrollPosition-sizeOfScroll);
	}
	function up() {
		setScroll (scrollPosition+sizeOfScroll);
	}
	function setScroll (scroll_num:Number,doRedrawWhileTweening:Boolean) {
		_tween=new Tween(container_mc, propertyToScroll, /*Elastic.easeOut*/Strong.easeOut, scrollPosition, scroll_num, 3, true);
		if (doRedrawWhileTweening!=false)
			_tween.onMotionChanged=Utils.createDelegate(this,redraw);
	}
	// resizes the object to the requested size.
	function setSize (w:Number,h:Number) {
		trace("CustomScrollBar setSize "+w+" x "+h);
		
		_xscale=_yscale=100;
		
		// set bg size
		//bg_mc._width=w;
		bg_mc._height=h;
	}
}
