﻿/**
	this file is part of SILEX
	SILEX : RIA developement tool - see http://silex-ria.org/

	SILEX is (c) 2004-2007 Alexandre Hoyau and is released under the GPL License:

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License (GPL)
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.
	
	To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/

/**
 *
 * Name : UiBase.as
 * Package : ui
 * Version : 0.8
 * Date :  2007-08-03
 */
	 
//import org.silex.core.Constants;
//import org.silex.core.Tooltip;
//import org.silex.ui.Interface;
import mx.events.EventDispatcher;
import org.silex.core.Utils;

/* listen to
core.Authentication
[Event("loginSuccess")]
[Event("logout")]
[Event("loginError")]

core.Application 
[Event("XMLLoaded")]
[Event("showContent")]
[Event("hideContent")]
[Event("onUnload")]

core.Layout
[Event("onDeeplink")]

ui.players.UiBase
[Event("onRelease")]
*/
[Event("loginSuccess")]
[Event("logout")]
[Event("loginError")]

[Event("XMLLoaded")]
[Event("showContent")]
[Event("hideContent")]
[Event("onUnload")]
[Event("onDeeplink")]

// defined in sub classes but common to all players
[Event("onPress")]
[Event("onRelease")]
[Event("onRollOver")]
[Event("onRollOut")]
[Event("onMouseMove")]

class org.silex.ui.UiBase extends mx.core.UIComponent implements org.silex.ui.Interface {

	//silex 
	var silexInstance/*:org.silex.core.Api*/ = null;
	// layout
	var layoutInstance/*:org.silex.core.Layout*/=null;
	//layer parent
	var layerInstance:MovieClip = null;
	//contener player
	var playerContainer:MovieClip = null;
	
	//arrays
	//editable properties
	var editableProperties:Array = null;		
	//available methods
    var availableMethods:Array = null;
    //available events
    var availableEvents:Array = null;
	//contextMenu
	var availableContextMenuMethods = null;
    //actions for a specific event
	var actions:Array = null;    

	// default value for width and height
	static var defaultSize:Number = 100;
	
	//tooltip
	//var playerTip:org.silex.core.Tooltip = null;
	var playerTip = null;
	var tooltip_str:String = null;
	
	//EventDispatcher
	var addEventListener:Function;
 	var removeEventListener:Function;
    var dispatchEvent:Function
	
	//others properties used by interface
	var playerName:String;
	var alternateText:String;
	var descriptionText:String;
	var tags:Array;
	// setter getter below - var visibleOutOfAdmin:Boolean = true; //displayed or not out of moderation	

	// temporary visiblility : _visible = visibleOutOfAdmin && temporarilyVisible
	var temporarilyVisible:Boolean = true;
	var temporarilyVisibleInAdminMode:Boolean = true;
	
	var _isEditable:Boolean; //used by set/get isEditable
	var selected:Boolean ;
	var focus:Boolean;
	//url
	var privateUrl_str:String = null;
	
    /**
	 * function constructor
	 * @return void
	 */
     public function UiBase()
     {
		 // workaround bug tilt in the upper left corner
		 //_x = _y = -5000;
		 
    	//silex instance
		this.silexInstance =_global.getSilex(this);
		// layout
		layoutInstance=silexInstance.application.getLayout(this);
		//layer 
		this.layerInstance = this._parent._parent;
		this.playerContainer = this._parent;
		//eventDispatcher
		EventDispatcher.initialize(this);
		//editableProperties
		this.editableProperties = new Array(); 
		//availableMethods
		this.availableMethods = new Array();
		//availablevents
	   	this.availableEvents = new Array();
		//availablevents
	   	this.availableContextMenuMethods = new Array();	
	   	//actions 
	   	this.actions = new Array();
		
		//tooltip 
		playerTip = this.attachMovie('it.sephiroth.tooltip','tooltip', this.getNextHighestDepth());
		
		//editable
		this._isEditable = false;
		//visible true by default
		this.visibleOutOfAdmin = true;
		
//		this.silexInstance.sequencer.doInNFrames(5,Utils.createDelegate(this,_onLoad));
		doLater(Utils.createDelegate(this,_onLoad));
//		_onLoad();
    }
	
	
	/**
	 * function _populateProperties. load all editable properties for Silex
	 * @return void
	 */
	function _populateProperties(){
		//center media /stage dimension
		var defaultX:Number = this.silexInstance.application.stageRect.right + this.silexInstance.application.stageRect.left; 
		var defaultY:Number = this.silexInstance.application.stageRect.bottom + this.silexInstance.application.stageRect.top;		
		//editableProperties for every player
		this.editableProperties.push (
			{ name :"playerName" ,		description:silexInstance.config.PROPERTIES_LABEL_PLAYER_NAME, 					type: silexInstance.config.PROPERTIES_TYPE_URL		, defaultValue: this.playerContainer.name_str				, 	isRegistered:true, group:"attributes" },
			{ name :"descriptionText" , description:silexInstance.config.PROPERTIES_LABEL_SEO_DESCRIPTION, 		type: silexInstance.config.PROPERTIES_TYPE_TEXT		, defaultValue: ""					, 	isRegistered:true, group:"parameters" },
			{ name :"tags" , description:silexInstance.config.PROPERTIES_LABEL_SEO_TAGS, 		type: silexInstance.config.PROPERTIES_TYPE_ARRAY		, defaultValue: []					, 	isRegistered:true, group:"parameters" },
			{ name :"width" ,			description:silexInstance.config.PROPERTIES_LABEL_WIDTH, 				type: silexInstance.config.PROPERTIES_TYPE_NUMBER		, defaultValue: "none"	, isRegistered:true 	, minValue:0,  	maxValue:5000, group:"attributes" },
			{ name :"height" , 			description:silexInstance.config.PROPERTIES_LABEL_HEIGHT, 				type: silexInstance.config.PROPERTIES_TYPE_NUMBER 		, defaultValue: ""	, isRegistered:true 	, minValue:0,  	maxValue:5000, group:"attributes" },
			{ name :"iconPageName" , 	description:silexInstance.config.PROPERTIES_LABEL_ICON_NAME_OF_TARGETED_PAGE,						type: silexInstance.config.PROPERTIES_TYPE_URL, 			defaultValue: "new page name"	, isRegistered:true, group:"navigation"  },
			{ name :"iconDeeplinkName" , 	description:silexInstance.config.PROPERTIES_LABEL_ICON_DEEPLINK_OF_TARGETED_PAGE,						type: silexInstance.config.PROPERTIES_TYPE_URL, 			defaultValue: ""	, isRegistered:true, group:"navigation"},
			{ name :"iconLayoutName" ,	description:silexInstance.config.PROPERTIES_LABEL_ICON_NAME_OF_LAYOUT,		type: silexInstance.config.PROPERTIES_TYPE_URL,	 		defaultValue:layoutInstance.layoutFileName		, isRegistered:true, group:"navigation" },
			{ name :"iconIsIcon" , 		description:silexInstance.config.PROPERTIES_LABEL_ICON_IS_ICON, 					type: silexInstance.config.PROPERTIES_TYPE_BOOLEAN, 			defaultValue: false		, 	isRegistered:true, group:"navigation" },
			{ name :"iconIsDefault" , 	description:silexInstance.config.PROPERTIES_LABEL_ICON_IS_DEFAULT_ICON, 			type: silexInstance.config.PROPERTIES_TYPE_BOOLEAN, 			defaultValue: false		, 	isRegistered:true, group:"navigation"  },
			{ name :"_alpha" , 			description:silexInstance.config.PROPERTIES_LABEL_OPACITY, 				type: silexInstance.config.PROPERTIES_TYPE_NUMBER 	, defaultValue: 100					, 	isRegistered:true,	minValue:0,  		maxValue:100, group:"attributes" },
			{ name :"_x" , 				description:silexInstance.config.PROPERTIES_LABEL_X_POSITION, 				type: silexInstance.config.PROPERTIES_TYPE_NUMBER 	, defaultValue: defaultX			, 	isRegistered:true,	minValue:-5000,  	maxValue:5000, group:"attributes" },
			{ name :"_y" , 				description:silexInstance.config.PROPERTIES_LABEL_Y_POSITION, 				type: silexInstance.config.PROPERTIES_TYPE_NUMBER 	, defaultValue: defaultY			, 	isRegistered:true,	minValue:-5000,  	maxValue:5000, group:"attributes" },
			{ name :"rotation" , 		description:silexInstance.config.PROPERTIES_LABEL_ROTATION, 			type: silexInstance.config.PROPERTIES_TYPE_NUMBER 	, defaultValue: 0					, 	isRegistered:true,	minValue:0, 		maxValue:359, group:"attributes" },
			{ name :"url" ,				description:silexInstance.config.PROPERTIES_LABEL_URL, 						type: silexInstance.config.PROPERTIES_TYPE_URL 		, defaultValue: this.playerContainer.media_str	, isRegistered:true, group:"attributes"	},
			{ name :"visibleOutOfAdmin" , 		description:silexInstance.config.PROPERTIES_LABEL_IS_VISIBLE, 				type: silexInstance.config.PROPERTIES_TYPE_BOOLEAN	, defaultValue: true				, 	isRegistered:true, group:"attributes" },
			{ name :"tooltipText" , 	description:silexInstance.config.PROPERTIES_LABEL_TOOLTIP, 					type: silexInstance.config.PROPERTIES_TYPE_TEXT	 	, defaultValue: undefined , isRegistered:true, group:"parameters"  },
			{ name :"tabEnabled" , 	description:silexInstance.config.PROPERTIES_LABEL_ENABLE_TABULATION,		type: silexInstance.config.PROPERTIES_TYPE_BOOLEAN 		, defaultValue: false	, isRegistered:true, group:"others" },
			{ name :"tabIndex" , 		description:silexInstance.config.PROPERTIES_LABEL_TABULATION_INDEX, 	type: silexInstance.config.PROPERTIES_TYPE_NUMBER 		, defaultValue: 1		, isRegistered:true , minValue:1,  	maxValue:5000, group:"others" }
		);
		if (playerContainer.isEditable != undefined) setEditable(playerContainer.isEditable);
	}
	/**
	 * function _populateEvents. load all events for Silex
	 * @return void
	 */
	function _populateEvents(){
		//availableEvents
		this.availableEvents.push(
			{modifier:"loginSuccess", 		description : "dispatched when login occures"},
			{modifier:"loginError", 		description : "dispatched when login failed"},
			{modifier:"logout", 		description : "dispatched when logout occures"},
			{modifier:"onPress", 		description : "on mouse press"},
			{modifier:"onRelease",		description : "on mouse release"},
			{modifier:"onPress", 		description : "on mouse press"},			
		    {modifier:"onRollOut", 		description : "on mouse rollout"},	
			{modifier:"onRollOver", 		description : "on mouse rollover"},			
			{modifier:"onMouseMove", 		description : "on mouse move"},
			{modifier:"XMLLoaded", 		description : "on XML loaded"},
			{modifier:"showContent", 	description : "show Content"},
			{modifier:"hideContent", 	description : "hide Content"},
			{modifier:"onUnload", 		description : "on Unload"},
			{modifier:"onDeeplink", 	description : "on Deeplink"}
        );
	}
	
	/**
	 * function _populateMethods. load all events for Silex
	 * @return void
	 */
	function _populateMethods(){
		
	}
	/**
	 * function _populateContextMenu. (right click)
	 * @return void
	 */
	function _populateContextMenu(){
		// defines contextMenu
		this.availableContextMenuMethods.push(
                { fct :"rotatePlayer",     description :silexInstance.config.UI_PLAYERS_CONTEXT_MENU_LABEL_RESET_ROTATION,         	param : 0},
                { fct :"changeZOrder",     description :silexInstance.config.UI_PLAYERS_CONTEXT_MENU_LABEL_ZORDER_DOWN,    			 	param : "down"},
                { fct :"changeZOrder",     description :silexInstance.config.UI_PLAYERS_CONTEXT_MENU_LABEL_ZORDER_UP,     				param : "up"},
                { fct :"changeZOrder",     description :silexInstance.config.UI_PLAYERS_CONTEXT_MENU_LABEL_ZORDER_TO_BOTTOM,     		param : "background"},
                { fct :"changeZOrder",     description :silexInstance.config.UI_PLAYERS_CONTEXT_MENU_LABEL_ZORDER_TO_TOP,         	param : "foreground"}
        );
		// init
		menu = _root.menu;
	}
		
	/** function _initialize. kept for legacy reasons. should be eliminated soon
	 * @return void
	 */
	function _initialize():Void{
	}
	
	/**
	 * function _initBeforeRegister. 
	 * @return void
	 */
	function _initBeforeRegister(){
		_initialize();
	}
	
	function _registerWithSilex(){
		//silexInstance.utils.trace("ui.players.UiBase _registerWithSilex ");
	    //register Silex variable for editableProperties
		//count objects in array
		if (this.silexInstance.isSilexServer==true){ // only in SILEX
			var len:Number = this.editableProperties.length;
			for (var i:Number = 0; i < len; i++)
			{
				//silexInstance.utils.trace("ui.players.UiBase _registerWithSilex "+i+" - "+editableProperties[i]+" - "+editableProperties[i].description+" - "+editableProperties[i].type);
				//get type 
				var type_str:String = getASType(this.editableProperties[i].type);
				//if registerable
				if(this.editableProperties[i].isRegistered){
					//silexInstance.utils.trace("BEFORE "+this.editableProperties[i].name+" - "+type_str);
					//if not Register silex variable
					if(!this.silexInstance.dynDataManager.registerVariable(this, this.editableProperties[i].name,type_str)){
						// silexInstance.utils.trace("AFTER false");
						//silexInstance.utils.trace("ui.players.UiBase _registerWithSilex "+i+" - "+editableProperties[i].defaultValue+" - "+editableProperties[i].description);
						//if not null apply defaults value
						if( this.editableProperties[i].defaultValue != undefined && this.editableProperties[i].defaultValue.toString() != ""){ 
							this[this.editableProperties[i].name] = this.editableProperties[i].defaultValue;
						}
					}
					//silexInstance.utils.trace("AFTER true");
				}else{
					if( this.editableProperties[i].defaultValue != ""){ 
						this[this.editableProperties[i].name] = this.editableProperties[i].defaultValue;
						//trace
						//silexInstance.utils.trace( 'ui.players.UiBase apply default values - prop :'+ this.editableProperties[i].name + ' - defaultvalue :' +  this.editableProperties[i].defaultValue )
					}
				}
			}
		}
		
		//actions register 
		this.silexInstance.dynDataManager.registerVariable(this,"actions","array");
		//register player in his layer
		layerInstance.registerPlayer(this);		
		
		
	}
	
	/**
	 * function _initAfterRegister. 
	 * @return void
	 */
	function _initAfterRegister(){
		//eventlistener layout
		layoutInstance.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_XMLLOADED,	Utils.createDelegate(this,_XMLLoaded));
		layoutInstance.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_CONTENT_HIDE_START,	Utils.createDelegate(this,_hideContentStart));
		layoutInstance.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_CONTENT_HIDE,	Utils.createDelegate(this,_hideContent));
		layoutInstance.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_CONTENT_SHOW,	Utils.createDelegate(this,_showContent));
		layoutInstance.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_CHILD_HIDE,	Utils.createDelegate(this,_hideChild));
		layoutInstance.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_CHILD_SHOW,	Utils.createDelegate(this,_showChild));

		silexInstance.authentication.addEventListener( "loginSuccess",	Utils.createDelegate(this, dispatch));
		silexInstance.authentication.addEventListener( "loginError",	Utils.createDelegate(this, dispatch));
		silexInstance.authentication.addEventListener( "logout",	Utils.createDelegate(this, dispatch));
	}

	
/*	public function onLoad(){
		//doLater(this, "_onLoad");
		this.silexInstance.sequencer.doInNFrames(1,Utils.createDelegate(this,_onLoad));
	}*/
	
	/**
	 * when movieclip is transformed to UIoObject
	 */
	 public function _onLoad(){
		// against the green rectangle when setFocus is called
		this.drawFocus=undefined;
		

		try {this._populateProperties();}catch (e:Error){silexInstance.utils.trace("ui.players.UiBase _onLoad - ERROR in _populateProperties");}
		try {this._populateEvents();}catch (e:Error){silexInstance.utils.trace("ui.players.UiBase _onLoad - ERROR in _populateEvents");}
		try {this._populateContextMenu();}catch (e:Error){silexInstance.utils.trace("ui.players.UiBase _onLoad - ERROR in _populateContextMenu");}
		try {this._initBeforeRegister();}catch (e:Error){silexInstance.utils.trace("ui.players.UiBase _onLoad - ERROR in _initBeforeRegister");}
		try {this._registerWithSilex();}catch (e:Error){silexInstance.utils.trace("ui.players.UiBase _onLoad - ERROR in _registerWithSilex");}
		try {this._initAfterRegister();}catch (e:Error){silexInstance.utils.trace("ui.players.UiBase _onLoad - ERROR in _initAfterRegister");}
		
				
		//loadmedia
		//this.loadMedia( this.url);		

		//redraw Player
		this.redraw();
		
				
		//once all players are loaded
		this.layerInstance.addEventListener('allPlayersLoaded', Utils.createDelegate(this,_onAllPlayerLoaded));
		//stage resize
		this.silexInstance.application.addEventListener('resize', Utils.createDelegate(this,_onResize))

		// start if content is allready visible
		// silexInstance.utils.trace('ui.UiBase _onLoad isContentVisible='+layoutInstance.isContentVisible)
		if (layoutInstance.isContentVisible == true){
			_showContent();
		}

		this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_ONLOAD ,target:this });
	}	
	
	function _onAllPlayerLoaded(){
		
	}
	
	function _onResize(){
	}
	
	/**
	 * events 
	 */
	 function _XMLLoaded(){
		 //dispatch
		 this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_XMLLOADED, target:this });
	 }
	 
	 
	 function _hideContentStart(){
		 //silexInstance.utils.trace('ui.UiBase _hideContentStart ')
		 //dispatch
		  this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_CONTENT_HIDE_START, target:this });
	 }
	 
	 function _hideContent(){
		 //silexInstance.utils.trace('ui.UiBase _hidecontent ')
		 //dispatch
		  this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_CONTENT_HIDE, target:this });
		  //on unload
		  this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_UNLOAD, target:this });
		  //Remove listener
		  unloadMedia();
		  onUnload();
	}
	 
	 
	 function _showContent(){
		 //silexInstance.utils.trace('ui.UiBase _showContent ')
		 //dispatch
		  this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_CONTENT_SHOW, target:this });
	 }
	function _showChild(){
		 //silexInstance.utils.trace('ui.UiBase _showChild ')
		 //dispatch
		  this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_CHILD_SHOW, target:this });
	 }
	function _hideChild(){
		 //silexInstance.utils.trace('ui.UiBase _hideChild ')
		 //dispatch
		  this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_CHILD_HIDE, target:this });
	 }
	 
	 
	 
	 

	 
	/**
     * initContextMenu
     * push item in contextmenu 
     */
    function initContextMenu(){
	   
	   // init context menu
		if (!menu) menu = _root.menu.copy();
		if (!menu) menu = new ContextMenu;
        menu.hideBuiltInItems();
        // parse all items of availableContextMenuMethods
        var len:Number = availableContextMenuMethods.length;
        for (var i:Number = 0; i<len; i++){
            //test si method exists ?
            if(availableContextMenuMethods[i].fct){
                // adds a reference to the player (this) in the item
                availableContextMenuMethods[i].player_ptr=this;
                // build a ContextMenuItem object based on the item
                var menuItem_cmi:ContextMenuItem = new ContextMenuItem(availableContextMenuMethods[i].description,Utils.createDelegate(availableContextMenuMethods[i],execContextMenuMethod));
                //push item in context menu
                menu.customItems.push( menuItem_cmi );
            }
        }
    }
    
	function removeContextMenu() {
		menu = _root.menu.copy();
	}
	
	function addContextMenu(){		
		
		for( var eachProp in  this.menu.builtInItems) {
			var propName = eachProp;
			var propValue = this.menu.builtInItems[propName];
			//silexInstance.utils.trace("ui.UiBase context menu : "+ propName + ": " + propValue);
			propValue.enabled = true;
		}	
	}
	
    
    
    function execContextMenuMethod(){
        this["player_ptr"].silexInstance.utils.trace('UiBase context menu exec   fct : ' + this["fct"] + ' param : ' + this["param"]);
        //appel fct
		if( this["param"] != null){
       		 this["player_ptr"][this["fct"]](this["param"]);
		}else{
			this["player_ptr"][this["fct"]]();
		}
    }
	 
	 
	 
	//rotation 
	function rotatePlayer(angle:Number):Void{
		this.rotation = angle;
	}
	 
	// change z-order
	function changeZOrder(positionZ:String):Void{
		//players
		var l:Number = this.layerInstance.players.length;
		//get player indice position
		var indice:Number = null;
		for(var i:Number =0; i<l; i ++){
			if( this.layerInstance.players[i]._name == this.playerContainer._name){
				indice = i;
			}
		}
		//if indice found
		if(indice != null){
			//action to do
			switch(positionZ){
				case 'down':
					//if player not the last
					if(indice != l){
						var temp:Object = this.layerInstance.players[indice];
						//insert position +2
						this.layerInstance.players.splice(indice+2, 0, temp );
						//delete indice
						this.layerInstance.players.splice(indice,1);						
					}					
					break;
					
				case 'up':
					////if player not the first
					if( indice >= 1 ){
						var temp:Object = this.layerInstance.players[indice];
						//delete indice
						this.layerInstance.players.splice(indice,1);	
						//insert position +2
						this.layerInstance.players.splice(indice-1, 0, temp );						
					}
					break;
					
				case 'background':
					//players
					if( indice != l ){
						var temp:Object = this.layerInstance.players[indice];
						//Delete
						this.layerInstance.players.splice(indice,1);
						//push last
						this.layerInstance.players.push(temp);
					}
					break;
					
				case 'foreground':
					//players
					if( indice != 0 ){
						var temp:Object = this.layerInstance.players[indice];
						//Delete
						this.layerInstance.players.splice(indice,1);
						//push last
						this.layerInstance.players.unshift(temp);
					}
					break;						
			}
			//Refresh
			this.layerInstance.refresh();
		}else{
			silexInstance.utils.trace('ui.players.UiBase - Bug dans ui.player.UiBase changeZOrder !',silexInstance.constants.DEBUG_LEVEL_WARNING);
		}
	}
	
	/*
	 * animation when player is set to editable
	 */
	
	function animation(){
		 this.attachMovie('animation', 'animation', this.getNextHighestDepth() );
		 animation.play();
	}
	
	
	
	/**
	 * function getAsType
	 * @return String
	 */
    function getASType(type_str:String):String
	{
		switch(type_str)
		{
			case "rich text":
			case "coordinates":
			case "text":
			case "color":
				return "string";
			case "boolean":
				return "boolean";
			case "number":			
				return "number";
		}
		return "string";
	}
	
	

	
   /**
	* function dispatch
	* @return void
	*/
   public function dispatch(eventObject:Object):Void{
		//dispatch Event
		dispatchEvent(eventObject);
		//if not editable
		if(!this._isEditable){
			//silex interpreter
			//boucle actions
		    var len:Number = this.actions.length;
			for (var i:Number = 0; i<len; i++){
				//object in array
				var obj:Object = this.actions[i];
				//if modifier == event
				if( this.actions[i].modifier == eventObject.type){
					//silex command
					var function_str:String = this.actions[i].functionName ;
					var command_str:String=function_str;
					//parameters in a string
					if (this.actions[i].parameters && this.actions[i].parameters.length>0){
						var parameters_str:String = this.actions[i].parameters.toString();
						// format the text with accessors etc
						//parameters_str=silexInstance.utils.revealAccessors(parameters_str,this);
						parameters_str=silexInstance.utils.revealWikiSyntax(parameters_str);
						command_str+=":"  + parameters_str;
					}
					//silex interpreter
					trace("executing command :" + this.actions[i].modifier + " " + command_str + " from " + this);
					this.silexInstance.interpreter.exec( command_str  ,this);
				}					
			}
		}
	}

	
	
	function get url():String{
		return privateUrl_str;
	}
	
	function set url(url_str:String){
		privateUrl_str=url_str;		
	}
	
	
	/**
	* tooTip
	*/
	function get tooltipText():String{
		return tooltip_str;
	}
	
	function set tooltipText(value_str:String){

		tooltip_str = value_str;		
	}
	
	/**
	 * setVisible
	 * player visible or not on scene
	 * encapsulate set isVisible
	 * @param 	Boolean		show/hide
	 * @return  void
	 */ 
	public function setVisible(showIt:Boolean):Void {
		isVisible = showIt;			
	}

	// **
	// Icons variables
	private var _iconIsIcon:Boolean=false;
	private var _iconIsDefault:Boolean=false;
	var iconPageName:String;
	var iconDeeplinkName:String;
	var iconLayoutName:String;
	var iconListener:Object;

	/**
	 * set/get iconIsIcon
	 * indicate if the player is an icon, i.e. if it is part of the navigation menu
	 * @param 	Boolean
	 * @return  void|Boolean
	 */ 
	function get iconIsIcon():Boolean
	{
		return _iconIsIcon;
	}
	function set iconIsIcon(isIcon_bool:Boolean)
	{
		//silexInstance.utils.trace("ui.players.UiBase iconIsIcon set to "+isIcon_bool);
		if (_iconIsIcon==isIcon_bool){
			// nothing changed => do nothing
			return;
		}
		_iconIsIcon=isIcon_bool;

		if (_iconIsIcon==true)
		{// iconIsIcon is set to true
			iconListener=new Object;
			iconListener.onRelease=Utils.createDelegate(this,openIcon);
			iconListener.onDeeplink=Utils.createDelegate(this,openIconOnDeeplink);
			//release
			this.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_RELEASE, iconListener);
			layoutInstance.addEventListener( silexInstance.config.UI_PLAYERS_EVENT_DEEPLINK, iconListener);
		}
		else
		{// iconIsIcon is set to false
			// supress the listener
			if (iconListener)
			{
				this.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_RELEASE, iconListener);
				layoutInstance.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_DEEPLINK, iconListener);
				iconListener=null;
				
			}
		}
	}
	function get iconIsDefault():Boolean{
		return _iconIsDefault;
	}
	function set iconIsDefault(isDefault:Boolean){
		if (isDefault!=_iconIsDefault){
			if (isDefault==true){
				layoutInstance.registerDefaultIcon(this);
			}
			else{
				layoutInstance.unRegisterDefaultIcon(this);
			}
			_iconIsDefault=isDefault;
		}
	}
	/**
	 * openIconOnDeeplink
	 * check if the deeplink corresponds to this icon
	 * - core.Layout::onDeeplink event
	 */ 
	function openIconOnDeeplink(ev) {
		silexInstance.utils.trace("ui.players.UiBase openIconOnDeeplink "+iconPageName+" - "+iconLayoutName+" - "+iconIsDefault);
		// does the deeplink corresponds to this icon?
		if (iconPageName && ev.deeplinkName==silexInstance.utils.cleanID(iconPageName)){
			//silexInstance.utils.trace(this.playerName + "ui.players.UiBase openIconOnDeeplink - the deeplink corresponds to this icon: "+ev.deeplinkName+" - "+ev.cleanSectionName);

			//dispatch
			this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_DEEPLINK, target:this });

			// do open
			openIcon();
		}
	}
	/**
	 * openIcon
	 * check if the icon may open a page and eventually call doOpenIcon to open it
	 */ 
	function openIcon()
	{
		//silexInstance.utils.trace("ui.players.UiBase openIcon "+iconPageName+" - "+iconLayoutName+" - "+iconIsDefault+" - "+iconDeeplinkName+" - isEditable:"+this._isEditable);
		// is it initialized?
		//if (iconPageName==undefined || iconLayoutName==undefined)
		//	return;
			
		// 
		if (!this._isEditable){
			var dirtyChildLayout=silexInstance.application.isAChildDirty(layoutInstance);
			if (dirtyChildLayout && silexInstance.authentication.isLoggedIn()){
				silexInstance.utils.confirm(silexInstance.utils.revealAccessors(silexInstance.config.QUIT_DIRTY_PAGE_WARNING,dirtyChildLayout),Utils.createDelegate(this,openIconConfirmCallback));
			}
			else{
				doOpenIcon();
			}
		}
	}
	function openIconConfirmCallback(doIt:Boolean){
		if (doIt) doOpenIcon();
	}
	/**
	 * doOpenIcon
	 * open the page associated with this icon
	 * - media::onRelease event
	 * - core.Layout::onDeeplink event
	 */ 
	function doOpenIcon()
	{
		var revealedPageName:String = silexInstance.utils.revealAccessors(iconPageName, this);
		var revealedDeeplinkName:String = revealedPageName;
		if (iconDeeplinkName != undefined && iconDeeplinkName != "") revealedDeeplinkName = silexInstance.utils.revealAccessors(iconDeeplinkName, this);
		var revealedLayoutName:String = silexInstance.utils.revealAccessors(iconLayoutName, this);
		
		if (iconPageName!=undefined && iconLayoutName!=undefined && layoutInstance!=undefined && revealedLayoutName!="" && revealedPageName!="" && revealedDeeplinkName!="")
		{
			silexInstance.application.openSection(revealedPageName, revealedLayoutName, layoutInstance, revealedDeeplinkName);
		}
		else
		{
			silexInstance.utils.trace("ui.players.UiBase doOpenIcon ERROR in the inputs: " + iconPageName + " - " + iconLayoutName + " - " + layoutInstance + " - " + revealedLayoutName + " - " + revealedPageName + " - " + revealedDeeplinkName);
			return;
		}
		
		// unselect selected icon of the layout
		if(layoutInstance.selectedIcon)
			layoutInstance.selectedIcon.selectIcon(false);

		// store selected icon in layout
		layoutInstance.selectedIcon=this;
		
		// call selectIcon for derived classes
		selectIcon(true);
	}
	/**
	 * selectIcon
	 * called by openIcon and core.application::openSection
	 * to be overriden by sub classes - mark the media as selected?
	 */
	function selectIcon(isSelected:Boolean){
		//silexInstance.utils.trace("ui.players.UiBase selectIcon "+iconPageName+" - "+iconLayoutName+" - "+iconIsDefault);
	}


	/**
	 * set/get rotation
	 * @param 	Number		show/hide
	 * @return  void|Number
	 */ 
	 
	public function set rotation( rotationNumber:Number):Void{
		this._rotation = Math.round(rotationNumber);
	}
	
	public function get rotation():Number{
		return this._rotation;
	}
		
	/**
	 * setEditable
	 * encapsulate set editable
	 * @param 	Boolean		moderation or not 
	 * @return  void
	 */
	public function setEditable(edit:Boolean):Void {
		//silexInstance.utils.trace("ui.players.UiBase setEditable "+edit);
		this.isEditable= edit;			
	}
	 
	 
	/**
	 * set/get isEditable
	 * @param 	Boolean		moderation or not
	 * @return  void|Boolean
	 */ 
	public function set isEditable(allowEdit:Boolean):Void {
		this._isEditable = allowEdit;
		if( !this._isEditable){
			//assign visibility out of moderation defined by visible 
			this.removeContextMenu()
			//
			this._visible = visibleOutOfAdmin && temporarilyVisible;	
		}else{
			this._visible = temporarilyVisibleInAdminMode;
			//show contextmenu 
			this.initContextMenu();
			//anim 
			this.animation();
		}
	}	
	
	public function get isEditable():Boolean{
		return this._isEditable;
	}
	
	
	/**
	 * set/get isVisible
	 * visibility used by tool boxes
	 * @param 	Boolean		show/hide
	 * @return  void|Boolean
	 */ 
	public function set isVisible(showIt:Boolean):Void {
		//silexInstance.utils.trace("ui.players.UiBase set isVisible "+showIt);
		this._visible = showIt;
		temporarilyVisible = showIt;
	}
	
	public function get isVisible():Boolean{
		return this._visible;
	}
	/**
	 * set/get visibleOutOfAdmin
	 * visibility when out of admin mode
	 * @param 	Boolean		show/hide
	 * @return  void|Boolean
	 */ 
	var _visibleOutOfAdmin:Boolean=true;
	public function set visibleOutOfAdmin(showIt:Boolean):Void {
		//silexInstance.utils.trace("ui.players.UiBase set isVisible "+showIt);
		_visibleOutOfAdmin=showIt;		
		if(_isEditable==true)
			_visible=temporarilyVisibleInAdminMode;
		else
			_visible=_visibleOutOfAdmin && temporarilyVisible;
	}
	
	public function get visibleOutOfAdmin():Boolean{
		return _visibleOutOfAdmin;
	}
	
		
	/**
	 * implementations interface 
	 * @param 	Boolean		focus true|false
	 * @return  void
	 */	
	public function setFocus(getFocusOrLooseIt:Boolean):Void {
			if(getFocusOrLooseIt){
				this.focus = true;
			}else{
				this.focus = false;
			}
	}
	
	/**
	 * implementations interface 
	 * @param 	Boolean		selected true|false
	 * @return  void
	 */
	public function setSelected(selectItOrDeselectIt:Boolean):Void {
			if(selectItOrDeselectIt){
				this.selected = true;
			}else{
				this.selected = false;
			}
	}
	
	/**
	 * implementations interface 
	 * @param 	String		media Url
	 * @return  void
	 */
	public function loadMedia(url:String):Void{
		//TODO : use methode globale
		//core.utils.loadMedia();
	}
	
	
	
	/**
	 * implementations interface 
	 * @return  void
	 */
	public function unloadMedia():Void {
		//once all players are loaded
		this.layerInstance.removeEventListener('allPlayersLoaded', Utils.createDelegate(this,_onAllPlayerLoaded));
		//stage resize
		this.silexInstance.application.removeEventListener('resize', Utils.createDelegate(this,_onResize))

		layoutInstance.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_XMLLOADED,	Utils.createDelegate(this,_XMLLoaded));
		layoutInstance.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_CONTENT_HIDE_START,	Utils.createDelegate(this,_hideContentStart));
		layoutInstance.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_CONTENT_HIDE,	Utils.createDelegate(this,_hideContent));
		layoutInstance.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_CONTENT_SHOW,	Utils.createDelegate(this,_showContent));
		layoutInstance.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_CHILD_HIDE,	Utils.createDelegate(this,_hideChild));
		layoutInstance.removeEventListener( silexInstance.config.UI_PLAYERS_EVENT_CHILD_SHOW,	Utils.createDelegate(this, _showChild));				
		
		
		silexInstance.authentication.removeEventListener( "loginSuccess",	Utils.createDelegate(this, dispatch));
		silexInstance.authentication.removeEventListener( "loginError",	Utils.createDelegate(this, dispatch));
		silexInstance.authentication.removeEventListener( "logout",	Utils.createDelegate(this, dispatch));
	}
	
	
	/** events **/
	
	/**
	 * function _onRelease
	 * @return 	void
	 */
	function _onRelease():Void{
		//dispatch
		this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_RELEASE ,target:this });
	}
	
	/**
	 * function _onPress
	 * @return 	void
	 */	
	function _onPress():Void{
		//dispatch
		this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_PRESS ,target:this });
	}
	
	/**
	 * function _onRollOver
	 * @return 	void
	 */
	function _onRollOver():Void{
		onMouseMove = _onMouseMove;
		//tooltip
		if(playerTip != null && tooltipText != undefined && tooltipText != '' ){
			//silexInstance.utils.trace("ui.players.Image playerTip draw "+ playerTip);
			//playerTip.drawTip();	
			this.playerTip.text = tooltipText;
			// show tip, interval, should follow mouse
			this.playerTip.show(500, true);
		}
		//dispatch
		this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_ROLLOVER ,target:this });
	}
	
	/**
	 * function _onRollOut
	 * @return 	void
	 */	
	function _onRollOut():Void{
		onMouseMove = undefined;
		//tooltip remove
		if(playerTip != null ){
			//silexInstance.utils.trace("ui.players.UiBase playerTip remove ");
			//playerTip.removeTip();
			 this.playerTip.hide(300)
		}
		//dispatch
		this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_ROLLOUT ,target:this });
	}
	
	
	/**
	 * function _onMouseMove
	 * @return 	void
	 */
	function _onMouseMove():Void{
		//gérer tooltip ?
		//if(true)silexInstance.utils.alert("_onMouseMove");
		//dispatch
		this.dispatch({type: silexInstance.config.UI_PLAYERS_EVENT_MOUSEMOVE ,target:this });
	}
	
	
		

	/* getSeoData
	 * return the seo data to be associated with this player
	 * to be overriden in derived class :
	 * @return	object with text (string), tags (array), description (string), links (object with link, title and description), htmlEquivalent (string), context (array)
	 */
	function getSeoData(url_str:String):Object {
		var res_obj:Object=new Object;
		// keywords
		//res_obj.text="test";
		
		// tags
		if (tags)
			res_obj.tags=tags;
		
		// description
		if (descriptionText)
			res_obj.description=descriptionText;
		
		// links
		if (iconIsIcon){
			res_obj.links=new Array;
			//res_obj.href+="<u>debug</u><br>layoutRelativePath="+layoutRelativePath+"<br>layoutPath="+layoutPath+"<br>iconPageName="+silex_ptr.utils.cleanID(player.iconPageName)+"<br>";
			// YES : res_obj.href+="<a href='"+layoutRelativePath+silex_ptr.config.id_site+"/"+layoutPath+"/"+silex_ptr.utils.cleanID(player.iconPageName)+"'><H2>"+player.iconPageName+"</H2><br>"+res_obj.description+"<br>";
			//// YES : v1beta4
			/*
			res_obj.href+="<a href='./"+silex_ptr.utils.cleanID(player.iconPageName)+"/'><H2>"+player.iconPageName+"</H2><br>"+res_obj.description+"<br>";
			if (_obj.htmlEquivalent) res_obj.href+=_obj.htmlEquivalent;
			res_obj.href+="</a><br>";
			*/
			res_obj.links.push({link:silexInstance.utils.cleanID(iconPageName),title:iconPageName,description:descriptionText});
		}
		
		// html equivalent
		// res_obj.htmlEquivalent=descriptionText;
		
		// context
		res_obj.context = getUiContext();
		
		//silexInstance.utils.trace("getSeoData : " + mx.data.binding.ObjectDumper.toString(res_obj));
		return res_obj;
	}
	
	
	
	/////////////////////////////////////////
	// UIComponent methods
	// available for 
	// * ActionScript
	// * SILEX (commands)
	/////////////////////////////////////////
    
    function setSize(w:Number, h:Number){
		//trace(this.playerName + " setSize. width : " + w + ", height : " + h);
        super.setSize(w, h);
    }
    
	function getGlobalCoordTL(){
		//trace(this.playerName + " getGlobalCoordTL");
		//trace(this.playerName + " _x : " + _x);
		// from global coords
		var coordTL:Object={x:_x,y:_y};
	
		// to global
		_parent.localToGlobal(coordTL);
		
		//trace("left:"+coordTL.x+",top:"+coordTL.y);
		return coordTL;
	}
	
	function getGlobalCoordBR(){
		//trace(this.playerName + " getGlobalCoordBR");
		// from global coords
		var coordBR:Object={x:width+_x,y:height+_y};
	
		// to global
		_parent.localToGlobal(coordBR);
		
		//trace("right:"+coordBR.x + ",bottom:"+coordBR.y);
		return coordBR;
	}
	
	
	// **
	// override abstract class :
	// setGlobalCoordTL sets coordinates of the media
	// it substracts the registration point coords to coord
	// and apply the new coordinates to the player
	function setGlobalCoordTL(coord:Object){
		//trace(this.playerName + " setGlobalCoordTL");
		// silexInstance.utils.trace("ui.players.Image setGlobalCoordTL ("+coord.x+","+coord.y+") in global");
		// back to local
		_parent.globalToLocal(coord);
		// silexInstance.utils.trace("ui.players.Image setGlobalCoordTL ("+coord.x+","+coord.y+") in player");
		
		// apply the new coordinates to the player
		_x = coord.x;
		_y = coord.y;
	}
	function setGlobalCoordBR(coord:Object){
		//trace(this.playerName + " setGlobalCoordBR");
		// silexInstance.utils.trace("ui.players.Image setGlobalCoordBR ("+coord.x+","+coord.y+") in global");
	
		// back to local
		_parent.globalToLocal(coord);
		// silexInstance.utils.trace("ui.players.Image setGlobalCoordBR ("+coord.x+","+coord.y+") in player");
		
		// apply the new coordinates to the player
		width=coord.x - _x;
		height=coord.y - _y;
	}
	
	function get width ():Number {
		//trace(this.playerName + " get width " + super.width);
		return super.width;
	}
	
	function set width (val:Number) {
		if (val == undefined) val = defaultSize;
		//trace(this.playerName + " set width " + val);
		setSize(val,height);
	}
	
	function get height ():Number {
		//trace(this.playerName + " get height " + super.height);
		return super.height;
	}
	function set height (val:Number) {
		if (val == undefined) val = defaultSize;
		//trace(this.playerName + " set height" + val);
		setSize(width,val);
	}
	/* getUiContext
	 * retrieve the context of this object 
	 * use the layer's players array
	 */
	function getUiContext():Array{
		var l:Number = this.layerInstance.players.length;
		for(var i:Number =0; i<l; i ++){
			if( this.layerInstance.players[i]._name == this.playerContainer._name){
				return this.layerInstance.players[i].contextAccepted.join(",");
			}
		}
		return null;
	}
}
